@@ -2,18 +2,21 @@
 // subject to the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/python.hpp>
+#include "boost_python.hpp"
 #include <libtorrent/alert.hpp>
 #include <libtorrent/alert_types.hpp>
+#include <libtorrent/piece_picker.hpp> // for piece_block
+#include <libtorrent/session_stats.hpp>
 #include <memory>
+#include "bytes.hpp"
 
 using namespace boost::python;
 using namespace libtorrent;
 
-std::string get_buffer(read_piece_alert const& rpa)
+bytes get_buffer(read_piece_alert const& rpa)
 {
-    return rpa.buffer ? std::string(rpa.buffer.get(), rpa.size)
-       : std::string();
+    return rpa.buffer ? bytes(rpa.buffer.get(), rpa.size)
+       : bytes();
 }
 
 tuple endpoint_to_tuple(tcp::endpoint const& ep)
@@ -21,6 +24,12 @@ tuple endpoint_to_tuple(tcp::endpoint const& ep)
     return boost::python::make_tuple(ep.address().to_string(), ep.port());
 }
 
+tuple endpoint_to_tuple(udp::endpoint const& ep)
+{
+    return boost::python::make_tuple(ep.address().to_string(), ep.port());
+}
+
+
 tuple peer_alert_ip(peer_alert const& pa)
 {
     return endpoint_to_tuple(pa.ip);
@@ -43,6 +52,11 @@ tuple incoming_connection_alert_ip(incoming_connection_alert const& ica)
     return endpoint_to_tuple(ica.ip);
 }
 
+tuple dht_outgoing_get_peers_alert_ip(dht_outgoing_get_peers_alert const& a)
+{
+    return endpoint_to_tuple(a.ip);
+}
+
 std::string external_ip_alert_ip(external_ip_alert const& eia)
 {
     return eia.external_address.to_string();
@@ -93,13 +107,107 @@ dict get_params(add_torrent_alert const& alert)
     return ret;
 }
 
+list dht_stats_active_requests(dht_stats_alert const& a)
+{
+   list result;
+
+   for (std::vector<dht_lookup>::const_iterator i = a.active_requests.begin();
+		i != a.active_requests.end(); ++i)
+   {
+		dict d;
+
+		d["type"] = i->type;
+		d["outstanding_requests"] = i->outstanding_requests;
+		d["timeouts"] = i->timeouts;
+		d["responses"] = i->responses;
+		d["branch_factor"] = i->branch_factor;
+		d["nodes_left"] = i->nodes_left;
+		d["last_sent"] = i->last_sent;
+		d["first_timeout"] = i->first_timeout;
+
+      result.append(d);
+   }
+   return result;
+}
+
+list dht_stats_routing_table(dht_stats_alert const& a)
+{
+   list result;
+
+   for (std::vector<dht_routing_bucket>::const_iterator i = a.routing_table.begin();
+		i != a.routing_table.end(); ++i)
+   {
+		dict d;
+
+		d["num_nodes"] = i->num_nodes;
+		d["num_replacements"] = i->num_replacements;
+
+      result.append(d);
+   }
+   return result;
+}
+
+dict dht_immutable_item(dht_immutable_item_alert const& alert)
+{
+    dict d;
+    d["key"] = alert.target.to_string();
+    d["value"] = alert.item.to_string();
+    return d;
+}
+
+dict dht_mutable_item(dht_mutable_item_alert const& alert)
+{
+    dict d;
+    d["key"] = std::string(alert.key.data(), alert.key.size());
+    d["value"] = alert.item.to_string();
+    d["signature"] = std::string(alert.signature.data(), alert.signature.size());
+    d["seq"] = alert.seq;
+    d["salt"] = alert.salt;
+    d["authoritative"] = alert.authoritative;
+    return d;
+}
+
+dict dht_put_item(dht_put_alert const& alert)
+{
+    dict d;
+    if (alert.target.is_all_zeros()) {
+        d["public_key"] = std::string(alert.public_key.data(), alert.public_key.size());
+        d["signature"] = std::string(alert.signature.data(), alert.signature.size());
+        d["seq"] = alert.seq;
+        d["salt"] = alert.salt;
+    } else {
+        d["target"] = alert.target.to_string();
+    }
+    return d;
+}
+
+dict session_stats_values(session_stats_alert const& alert)
+{
+    std::vector<stats_metric> map = session_stats_metrics();
+    dict d;
+
+    for (std::vector<stats_metric>::const_iterator i = map.begin();
+       i != map.end(); ++i)
+    {
+        d[i->name] = alert.values[i->value_index];
+    }
+    return d;
+}
 
 void bind_alert()
 {
     using boost::noncopyable;
+#ifndef TORRENT_NO_DEPRECATE
+    typedef boost::shared_ptr<alert> alert_holder;
+#if BOOST_VERSION >= 106000
+    register_ptr_to_python<boost::shared_ptr<alert> >();
+#endif
+#else
+    typedef alert alert_holder;
+#endif
 
     {
-        scope alert_scope = class_<alert, boost::shared_ptr<alert>, noncopyable >("alert", no_init)
+        scope alert_scope = class_<alert, alert_holder, noncopyable >("alert", no_init)
             .def("message", &alert::message)
             .def("what", &alert::what)
             .def("category", &alert::category)
@@ -132,6 +240,9 @@ void bind_alert()
             .value("ip_block_notification", alert::ip_block_notification)
             .value("performance_warning", alert::performance_warning)
             .value("stats_notification", alert::stats_notification)
+            .value("session_log_notification", alert::session_log_notification)
+            .value("torrent_log_notification", alert::torrent_log_notification)
+            .value("peer_log_notification", alert::peer_log_notification)
             // deliberately not INT_MAX. Arch linux crash while throwing an exception
             .value("all_categories", (alert::category_t)0xfffffff)
             ;
@@ -145,7 +256,10 @@ void bind_alert()
 
     class_<tracker_alert, bases<torrent_alert>, noncopyable>(
         "tracker_alert", no_init)
+#ifndef TORRENT_NO_DEPRECATE
         .def_readonly("url", &tracker_alert::url)
+#endif
+        .def("tracker_url", &tracker_alert::tracker_url)
         ;
 
     class_<torrent_added_alert, bases<torrent_alert>, noncopyable>(
@@ -154,7 +268,7 @@ void bind_alert()
 
     class_<torrent_removed_alert, bases<torrent_alert>, noncopyable>(
         "torrent_removed_alert", no_init)
-        .def_readonly("info_hash", &torrent_removed_alert::info_hash) 
+        .def_readonly("info_hash", &torrent_removed_alert::info_hash)
         ;
 
     class_<read_piece_alert, bases<torrent_alert>, noncopyable>(
@@ -171,7 +285,10 @@ void bind_alert()
     ;
     class_<tracker_error_alert, bases<tracker_alert>, noncopyable>(
         "tracker_error_alert", no_init)
+#ifndef TORRENT_NO_DEPRECATE
         .def_readonly("msg", &tracker_error_alert::msg)
+#endif
+        .def("error_message", &tracker_error_alert::error_message)
         .def_readonly("times_in_row", &tracker_error_alert::times_in_row)
         .def_readonly("status_code", &tracker_error_alert::status_code)
         .def_readonly("error", &tracker_error_alert::error)
@@ -236,14 +353,19 @@ void bind_alert()
 
     class_<block_downloading_alert, bases<peer_alert>, noncopyable>(
         "block_downloading_alert", no_init)
+#ifndef TORRENT_NO_DEPRECATE
         .def_readonly("peer_speedmsg", &block_downloading_alert::peer_speedmsg)
+#endif
         .def_readonly("block_index", &block_downloading_alert::block_index)
         .def_readonly("piece_index", &block_downloading_alert::piece_index)
         ;
 
     class_<storage_moved_alert, bases<torrent_alert>, noncopyable>(
         "storage_moved_alert", no_init)
+#ifndef TORRENT_NO_DEPRECATE
         .def_readonly("path", &storage_moved_alert::path)
+#endif
+        .def("storage_path", &storage_moved_alert::storage_path)
         ;
 
     class_<storage_moved_failed_alert, bases<torrent_alert>, noncopyable>(
@@ -264,15 +386,19 @@ void bind_alert()
 
     class_<url_seed_alert, bases<torrent_alert>, noncopyable>(
         "url_seed_alert", no_init)
+#ifndef TORRENT_NO_DEPRECATE
         .def_readonly("url", &url_seed_alert::url)
         .def_readonly("msg", &url_seed_alert::msg)
+#endif
+        .def("server_url", &url_seed_alert::server_url)
         ;
 
     class_<file_error_alert, bases<torrent_alert>, noncopyable>(
         "file_error_alert", no_init)
-        .def_readonly("file", &file_error_alert::file)
         .def_readonly("error", &file_error_alert::error)
+        .def("filename", &file_error_alert::filename)
 #ifndef TORRENT_NO_DEPRECATE
+        .def_readonly("file", &file_error_alert::file)
         .def_readonly("msg", &file_error_alert::msg)
 #endif
         ;
@@ -286,7 +412,10 @@ void bind_alert()
     class_<listen_failed_alert, bases<alert>, noncopyable>(
         "listen_failed_alert", no_init)
         .def_readonly("endpoint", &listen_failed_alert::endpoint)
+        .def("listen_interface", &listen_failed_alert::listen_interface)
         .def_readonly("error", &listen_failed_alert::error)
+        .def_readonly("operation", &listen_failed_alert::operation)
+        .def_readonly("sock_type", &listen_failed_alert::sock_type)
         ;
 
     class_<listen_succeeded_alert, bases<alert>, noncopyable>(
@@ -315,6 +444,8 @@ void bind_alert()
         .def_readonly("map_type", &portmap_alert::map_type)
         ;
 
+#ifndef TORRENT_DISABLE_LOGGING
+
     class_<portmap_log_alert, bases<alert>, noncopyable>(
         "portmap_log_alert", no_init)
         .def_readonly("map_type", &portmap_log_alert::map_type)
@@ -324,6 +455,8 @@ void bind_alert()
 #endif
         ;
 
+#endif // TORRENT_DISABLE_LOGGING
+
     class_<fastresume_rejected_alert, bases<torrent_alert>, noncopyable>(
         "fastresume_rejected_alert", no_init)
         .def_readonly("error", &fastresume_rejected_alert::error)
@@ -345,7 +478,10 @@ void bind_alert()
 
     class_<scrape_failed_alert, bases<tracker_alert>, noncopyable>(
         "scrape_failed_alert", no_init)
+#ifndef TORRENT_NO_DEPRECATE
         .def_readonly("msg", &scrape_failed_alert::msg)
+#endif
+        .def("error_message", &scrape_failed_alert::error_message)
         ;
 
     class_<udp_error_alert, bases<alert>, noncopyable>(
@@ -372,7 +508,10 @@ void bind_alert()
     class_<file_renamed_alert, bases<torrent_alert>, noncopyable>(
         "file_renamed_alert", no_init)
         .def_readonly("index", &file_renamed_alert::index)
+#ifndef TORRENT_NO_DEPRECATE
         .def_readonly("name", &file_renamed_alert::name)
+#endif
+        .def("new_name", &file_renamed_alert::new_name)
         ;
 
     class_<file_rename_failed_alert, bases<torrent_alert>, noncopyable>(
@@ -500,13 +639,17 @@ void bind_alert()
         .value("upload_payload", stats_alert::upload_payload)
         .value("upload_protocol", stats_alert::upload_protocol)
         .value("upload_ip_protocol", stats_alert::upload_ip_protocol)
+#ifndef TORRENT_NO_DEPRECATE
         .value("upload_dht_protocol", stats_alert::upload_dht_protocol)
         .value("upload_tracker_protocol", stats_alert::upload_tracker_protocol)
+#endif
         .value("download_payload", stats_alert::download_payload)
         .value("download_protocol", stats_alert::download_protocol)
         .value("download_ip_protocol", stats_alert::download_ip_protocol)
+#ifndef TORRENT_NO_DEPRECATE
         .value("download_dht_protocol", stats_alert::download_dht_protocol)
         .value("download_tracker_protocol", stats_alert::download_tracker_protocol)
+#endif
     ;
 
     class_<anonymous_mode_alert, bases<torrent_alert>, noncopyable>(
@@ -540,4 +683,66 @@ void bind_alert()
         .def_readonly("old_ih", &torrent_update_alert::old_ih)
         .def_readonly("new_ih", &torrent_update_alert::new_ih)
         ;
+
+    class_<dht_outgoing_get_peers_alert, bases<alert>, noncopyable>(
+       "dht_outgoing_get_peers_alert", no_init)
+        .def_readonly("info_hash", &dht_outgoing_get_peers_alert::info_hash)
+        .def_readonly("obfuscated_info_hash", &dht_outgoing_get_peers_alert::obfuscated_info_hash)
+        .add_property("ip", &dht_outgoing_get_peers_alert_ip)
+        ;
+
+#ifndef TORRENT_DISABLE_LOGGING
+
+    class_<log_alert, bases<alert>, noncopyable>(
+       "log_alert", no_init)
+        .def("msg", &log_alert::msg)
+        ;
+
+    class_<torrent_log_alert, bases<torrent_alert>, noncopyable>(
+       "torrent_log_alert", no_init)
+        .def("msg", &torrent_log_alert::msg)
+        ;
+
+    class_<peer_log_alert, bases<peer_alert>, noncopyable>(
+       "peer_log_alert", no_init)
+        .def("msg", &peer_log_alert::msg)
+        ;
+
+    class_<picker_log_alert, bases<peer_alert>, noncopyable>(
+       "picker_log_alert", no_init)
+        .add_property("picker_flags", &picker_log_alert::picker_flags)
+        .def("blocks", &picker_log_alert::blocks)
+        ;
+
+#endif // TORRENT_DISABLE_LOGGING
+
+    class_<lsd_error_alert, bases<alert>, noncopyable>(
+       "lsd_error_alert", no_init)
+        .def_readonly("error", &lsd_error_alert::error)
+        ;
+
+    class_<dht_stats_alert, bases<alert>, noncopyable>(
+       "dht_stats_alert", no_init)
+       .add_property("active_requests", &dht_stats_active_requests)
+       .add_property("routing_table", &dht_stats_routing_table)
+        ;
+
+    class_<dht_immutable_item_alert, bases<alert>, noncopyable>(
+       "dht_immutable_item_alert", no_init)
+        .add_property("item", &dht_immutable_item)
+        ;
+
+    class_<dht_mutable_item_alert, bases<alert>, noncopyable>(
+       "dht_mutable_item_alert", no_init)
+        .add_property("item", &dht_mutable_item)
+        ;
+
+    class_<dht_put_alert, bases<alert>, noncopyable>(
+       "dht_put_alert", no_init)
+        .add_property("item", &dht_put_item)
+        ;
+    class_<session_stats_alert, bases<alert>, noncopyable>(
+        "session_stats_alert", no_init)
+        .add_property("values", &session_stats_values)
+        ;
 }
