@@ -2,19 +2,20 @@
 // subject to the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/python.hpp>
 #include <list>
 #include <string>
 #include <libtorrent/session.hpp>
-#include <libtorrent/settings.hpp> // for bencode_map_entry
 #include <libtorrent/storage.hpp>
 #include <libtorrent/error_code.hpp>
 #include <libtorrent/ip_filter.hpp>
 #include <libtorrent/disk_io_thread.hpp>
+#include <libtorrent/aux_/session_settings.hpp>
 #include <libtorrent/extensions.hpp>
-#include <libtorrent/lazy_entry.hpp>
+#include <libtorrent/bdecode.hpp>
 #include <libtorrent/bencode.hpp>
 #include <libtorrent/aux_/session_impl.hpp> // for settings_map()
+#include <libtorrent/torrent_info.hpp>
+#include <libtorrent/kademlia/item.hpp> // for sign_mutable_item
 
 #include <libtorrent/extensions/lt_trackers.hpp>
 #include <libtorrent/extensions/metadata_transfer.hpp>
@@ -25,12 +26,20 @@
 #include "gil.hpp"
 #include "bytes.hpp"
 
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
+#include "boost_python.hpp"
+
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
 using namespace boost::python;
 using namespace libtorrent;
+namespace lt = libtorrent;
 
 namespace
 {
-    void listen_on(session& s, int min_, int max_, char const* interface, int flags)
+#ifndef TORRENT_NO_DEPRECATE
+    void listen_on(lt::session& s, int min_, int max_, char const* interface, int flags)
     {
         allow_threading_guard guard;
         error_code ec;
@@ -39,31 +48,35 @@ namespace
         if (ec) throw libtorrent_exception(ec);
 #endif
     }
+#endif
 
-    void outgoing_ports(session& s, int _min, int _max)
+    void outgoing_ports(lt::session& s, int _min, int _max)
     {
         allow_threading_guard guard;
-        session_settings settings = s.settings();
-        settings.outgoing_ports = std::make_pair(_min, _max);
-        s.set_settings(settings);
+        settings_pack p;
+        p.set_int(settings_pack::outgoing_port, _min);
+        p.set_int(settings_pack::num_outgoing_ports, _max - _min);
+        s.apply_settings(p);
         return;
     }
 #ifndef TORRENT_DISABLE_DHT
-    void add_dht_node(session& s, tuple n)
+    void add_dht_node(lt::session& s, tuple n)
     {
         std::string ip = extract<std::string>(n[0]);
         int port = extract<int>(n[1]);
+        allow_threading_guard guard;
         s.add_dht_node(std::make_pair(ip, port));
     }
 
-    void add_dht_router(session& s, std::string router_, int port_)
+    void add_dht_router(lt::session& s, std::string router_, int port_)
     {
         allow_threading_guard guard;
         return s.add_dht_router(std::make_pair(router_, port_));
     }
-#endif
 
-    void add_extension(session& s, object const& e)
+#endif // TORRENT_DISABLE_DHT
+
+    void add_extension(lt::session& s, object const& e)
     {
 #ifndef TORRENT_DISABLE_EXTENSIONS
        if (!extract<std::string>(e).check()) return;
@@ -85,89 +98,101 @@ namespace
 #endif // TORRENT_DISABLE_EXTENSIONS
     }
 
-	void session_set_settings(session& ses, dict const& sett_dict)
+	void make_settings_pack(lt::settings_pack& p, dict const& sett_dict)
 	{
-		bencode_map_entry* map;
-		int len;
-		boost::tie(map, len) = aux::settings_map();
-
-		session_settings sett;
-		for (int i = 0; i < len; ++i)
+		list iterkeys = (list)sett_dict.keys();
+		for (int i = 0; i < boost::python::len(iterkeys); i++)
 		{
-			if (!sett_dict.has_key(map[i].name)) continue;
+			std::string key = extract<std::string>(iterkeys[i]);
 
-			void* dest = ((char*)&sett) + map[i].offset;
-			char const* name = map[i].name;
-			switch (map[i].type)
+			int sett = setting_by_name(key);
+			if (sett == 0) continue;
+
+			TORRENT_TRY
 			{
-				case std_string:
-					*((std::string*)dest) = extract<std::string>(sett_dict[name]);
-					break;
-				case character:
-					*((char*)dest) = extract<char>(sett_dict[name]);
-					break;
-				case boolean:
-					*((bool*)dest) = extract<bool>(sett_dict[name]);
-					break;
-				case integer:
-					*((int*)dest) = extract<int>(sett_dict[name]);
-					break;
-				case floating_point:
-					*((float*)dest) = extract<float>(sett_dict[name]);
-					break;
+				object value = sett_dict[key];
+				switch (sett & settings_pack::type_mask)
+				{
+					case settings_pack::string_type_base:
+						p.set_str(sett, extract<std::string>(value));
+						break;
+					case settings_pack::int_type_base:
+						p.set_int(sett, extract<int>(value));
+						break;
+					case settings_pack::bool_type_base:
+						p.set_bool(sett, extract<bool>(value));
+						break;
+				}
 			}
+			TORRENT_CATCH(...) {}
 		}
-
-		if (sett_dict.has_key("outgoing_port"))
-			sett.outgoing_ports.first = extract<int>(sett_dict["outgoing_port"]);
-		if (sett_dict.has_key("num_outgoing_ports"))
-			sett.outgoing_ports.second = sett.outgoing_ports.first + extract<int>(sett_dict["num_outgoing_ports"]);
-
-		ses.set_settings(sett);
 	}
 
-	dict session_get_settings(session const& ses)
+	boost::shared_ptr<lt::session> make_session(boost::python::dict sett, int flags)
 	{
-		session_settings sett;
+		settings_pack p;
+		make_settings_pack(p, sett);
+		return boost::make_shared<lt::session>(p, flags);
+	}
+
+#ifndef TORRENT_NO_DEPRECATE
+	void session_set_settings(lt::session& ses, object const& sett)
+	{
+		extract<session_settings> old_settings(sett);
+		if (old_settings.check())
 		{
 			allow_threading_guard guard;
-			sett = ses.settings();
+			ses.set_settings(old_settings);
 		}
-		dict sett_dict;
-		bencode_map_entry* map;
-		int len;
-		boost::tie(map, len) = aux::settings_map();
-		for (int i = 0; i < len; ++i)
+		else
 		{
-			void const* dest = ((char const*)&sett) + map[i].offset;
-			char const* name = map[i].name;
-			switch (map[i].type)
-			{
-				case std_string:
-					sett_dict[name] = *((std::string const*)dest);
-					break;
-				case character:
-					sett_dict[name] = *((char const*)dest);
-					break;
-				case boolean:
-					sett_dict[name] = *((bool const*)dest);
-					break;
-				case integer:
-					sett_dict[name] = *((int const*)dest);
-					break;
-				case floating_point:
-					sett_dict[name] = *((float const*)dest);
-					break;
-			}
+			settings_pack p;
+			make_settings_pack(p, extract<dict>(sett));
+			allow_threading_guard guard;
+			ses.apply_settings(p);
 		}
-		sett_dict["outgoing_port"] = sett.outgoing_ports.first;
-		sett_dict["num_outgoing_ports"] = sett.outgoing_ports.second - sett.outgoing_ports.first + 1;
-		return sett_dict;
+	}
+#endif
+
+	void session_apply_settings(lt::session& ses, dict const& sett_dict)
+	{
+		settings_pack p;
+		make_settings_pack(p, sett_dict);
+		allow_threading_guard guard;
+		ses.apply_settings(p);
+	}
+
+	dict session_get_settings(lt::session const& ses)
+	{
+		settings_pack sett;
+		{
+			allow_threading_guard guard;
+			sett = ses.get_settings();
+		}
+		dict ret;
+		for (int i = settings_pack::string_type_base;
+			i < settings_pack::max_string_setting_internal; ++i)
+		{
+			ret[name_for_setting(i)] = sett.get_str(i);
+		}
+
+		for (int i = settings_pack::int_type_base;
+			i < settings_pack::max_int_setting_internal; ++i)
+		{
+			ret[name_for_setting(i)] = sett.get_int(i);
+		}
+
+		for (int i = settings_pack::bool_type_base;
+			i < settings_pack::max_bool_setting_internal; ++i)
+		{
+			ret[name_for_setting(i)] = sett.get_bool(i);
+		}
+		return ret;
 	}
 
 #ifndef BOOST_NO_EXCEPTIONS
 #ifndef TORRENT_NO_DEPRECATE
-    torrent_handle add_torrent_depr(session& s, torrent_info const& ti
+    torrent_handle add_torrent_depr(lt::session& s, torrent_info const& ti
         , std::string const& save, entry const& resume
         , storage_mode_t storage_mode, bool paused)
     {
@@ -180,9 +205,18 @@ namespace
 
     void dict_to_add_torrent_params(dict params, add_torrent_params& p)
     {
-        // torrent_info objects are always held by an intrusive_ptr in the python binding
+        // torrent_info objects are always held by a shared_ptr in the python binding
         if (params.has_key("ti") && params.get("ti") != boost::python::object())
-            p.ti = extract<boost::intrusive_ptr<torrent_info> >(params["ti"]);
+        {
+           // make a copy here. We don't want to end up holding a python-owned
+           // object inside libtorrent. If the last reference goes out of scope
+           // on the C++ side, it will end up freeing the python object
+           // without holding the GIL and likely crash.
+           // https://mail.python.org/pipermail/cplusplus-sig/2007-June/012130.html
+           p.ti = boost::make_shared<torrent_info>(
+              extract<torrent_info const&>(params["ti"]));
+        }
+
 
         if (params.has_key("info_hash"))
             p.info_hash = sha1_hash(bytes(extract<bytes>(params["info_hash"])).arr);
@@ -261,7 +295,7 @@ namespace
 namespace
 {
 
-    torrent_handle add_torrent(session& s, dict params)
+    torrent_handle add_torrent(lt::session& s, dict params)
     {
         add_torrent_params p;
         dict_to_add_torrent_params(params, p);
@@ -276,7 +310,7 @@ namespace
 #endif
     }
 
-    void async_add_torrent(session& s, dict params)
+    void async_add_torrent(lt::session& s, dict params)
     {
         add_torrent_params p;
         dict_to_add_torrent_params(params, p);
@@ -286,6 +320,7 @@ namespace
         s.async_add_torrent(p);
     }
 
+#ifndef TORRENT_NO_DEPRECATE
     void dict_to_feed_settings(dict params, feed_settings& feed)
     {
         if (params.has_key("auto_download"))
@@ -298,7 +333,7 @@ namespace
             dict_to_add_torrent_params(dict(params["add_args"]), feed.add_args);
     }
 
-    feed_handle add_feed(session& s, dict params)
+    feed_handle add_feed(lt::session& s, dict params)
     {
         feed_settings feed;
         // this static here is a bit of a hack. It will
@@ -367,25 +402,37 @@ namespace
         return ret;
     }
 
-    void start_natpmp(session& s)
+    void start_natpmp(lt::session& s)
     {
         allow_threading_guard guard;
         s.start_natpmp();
     }
 
-    void start_upnp(session& s)
+    void start_upnp(lt::session& s)
     {
         allow_threading_guard guard;
         s.start_upnp();
     }
+#endif // TORRENT_NO_DEPRECATE
 
-    alert const* wait_for_alert(session& s, int ms)
+#ifndef TORRENT_NO_DEPRECATE
+    boost::shared_ptr<alert>
+#else
+    alert const*
+#endif
+    wait_for_alert(lt::session& s, int ms)
     {
         allow_threading_guard guard;
-        return s.wait_for_alert(milliseconds(ms));
+        alert const* a = s.wait_for_alert(milliseconds(ms));
+#ifndef TORRENT_NO_DEPRECATE
+        if (a == NULL) return boost::shared_ptr<alert>();
+        return boost::shared_ptr<alert>(a->clone().release());
+#else
+        return a;
+#endif
     }
 
-    list get_torrents(session& s)
+    list get_torrents(lt::session& s)
     {
         list ret;
         std::vector<torrent_handle> torrents;
@@ -401,6 +448,21 @@ namespace
         return ret;
     }
 
+    cache_status get_cache_info1(lt::session& s, torrent_handle h, int flags)
+    {
+       cache_status ret;
+       s.get_cache_info(&ret, h, flags);
+       return ret;
+    }
+
+#ifndef TORRENT_NO_DEPRECATE
+    cache_status get_cache_status(lt::session& s)
+    {
+       cache_status ret;
+       s.get_cache_info(&ret);
+       return ret;
+    }
+
     dict get_utp_stats(session_status const& st)
     {
         dict ret;
@@ -412,46 +474,35 @@ namespace
         return ret;
     }
 
-    list get_cache_info(session& ses, bytes const& b)
+    list get_cache_info2(lt::session& ses, sha1_hash ih)
     {
-        sha1_hash ih(b.arr);
-        std::vector<cached_piece_info> ret;
+       std::vector<cached_piece_info> ret;
 
-        {
-           allow_threading_guard guard;
-           ses.get_cache_info(ih, ret);
-        }
+       {
+          allow_threading_guard guard;
+          ses.get_cache_info(ih, ret);
+       }
 
-        list pieces;
-        ptime now = time_now();
-        for (std::vector<cached_piece_info>::iterator i = ret.begin()
-           , end(ret.end()); i != end; ++i)
-        {
-            dict d;
-            d["piece"] = i->piece;
-            d["last_use"] = total_milliseconds(now - i->last_use) / 1000.f;
-            d["next_to_hash"] = i->next_to_hash;
-            d["kind"] = i->kind;
-            pieces.append(d);
-        }
-        return pieces;
+       list pieces;
+       ptime now = time_now();
+       for (std::vector<cached_piece_info>::iterator i = ret.begin()
+          , end(ret.end()); i != end; ++i)
+       {
+          dict d;
+          d["piece"] = i->piece;
+          d["last_use"] = total_milliseconds(now - i->last_use) / 1000.f;
+          d["next_to_hash"] = i->next_to_hash;
+          d["kind"] = i->kind;
+          pieces.append(d);
+       }
+       return pieces;
     }
 
-#ifndef TORRENT_DISABLE_GEO_IP
-    void load_asnum_db(session& s, std::string file)
-    {
-        allow_threading_guard guard;
-        s.load_asnum_db(file.c_str());
-    }
-
-    void load_country_db(session& s, std::string file)
-    {
-        allow_threading_guard guard;
-        s.load_country_db(file.c_str());
-    }
+    void load_asnum_db(lt::session& s, std::string file) {}
+    void load_country_db(lt::session& s, std::string file) {}
 #endif
 
-    entry save_state(session const& s, boost::uint32_t flags)
+    entry save_state(lt::session const& s, boost::uint32_t flags)
     {
         allow_threading_guard guard;
         entry e;
@@ -459,7 +510,8 @@ namespace
         return e;
     }
 
-    object pop_alert(session& ses)
+#ifndef TORRENT_NO_DEPRECATE
+    object pop_alert(lt::session& ses)
     {
         std::auto_ptr<alert> a;
         {
@@ -470,45 +522,104 @@ namespace
         return object(boost::shared_ptr<alert>(a.release()));
     }
 
-    list pop_alerts(session& ses)
+    list pop_alerts(lt::session& ses)
     {
-        std::deque<alert*> alerts;
+        std::vector<alert*> alerts;
         {
             allow_threading_guard guard;
             ses.pop_alerts(&alerts);
         }
 
         list ret;
-        for (std::deque<alert*>::iterator i = alerts.begin()
+        for (std::vector<alert*>::iterator i = alerts.begin()
             , end(alerts.end()); i != end; ++i)
         {
-            ret.append(boost::shared_ptr<alert>(*i));
+            ret.append(boost::shared_ptr<alert>((*i)->clone().release()));
         }
         return ret;
     }
+#else
+    list pop_alerts(lt::session& ses)
+    {
+        std::vector<alert*> alerts;
+        {
+            allow_threading_guard guard;
+            ses.pop_alerts(&alerts);
+        }
 
-    void load_state(session& ses, entry const& st)
-	 {
-		 allow_threading_guard guard;
+        list ret;
+        for (std::vector<alert*>::iterator i = alerts.begin()
+            , end(alerts.end()); i != end; ++i)
+        {
+            ret.append(boost::python::ptr(*i));
+        }
+        return ret;
+    }
+#endif
 
-		 std::vector<char> buf;
-		 bencode(std::back_inserter(buf), st);
-		 lazy_entry e;
-		 error_code ec;
-		 lazy_bdecode(&buf[0], &buf[0] + buf.size(), e, ec);
-		 TORRENT_ASSERT(!ec);
-		 ses.load_state(e);
-	 }
+	void load_state(lt::session& ses, entry const& st, boost::uint32_t flags)
+	{
+		allow_threading_guard guard;
+
+		std::vector<char> buf;
+		bencode(std::back_inserter(buf), st);
+		bdecode_node e;
+		error_code ec;
+		bdecode(&buf[0], &buf[0] + buf.size(), e, ec);
+		TORRENT_ASSERT(!ec);
+		ses.load_state(e, flags);
+	}
+
+#ifndef TORRENT_DISABLE_DHT
+    void dht_get_mutable_item(lt::session& ses, std::string key, std::string salt)
+    {
+        TORRENT_ASSERT(key.size() == 32);
+        boost::array<char, 32> public_key;
+        std::copy(key.begin(), key.end(), public_key.begin());
+        ses.dht_get_item(public_key, salt);
+    }
+
+    void put_string(entry& e, boost::array<char, 64>& sig, boost::uint64_t& seq,
+                    std::string const& salt, std::string public_key, std::string private_key,
+                    std::string data)
+    {
+        using libtorrent::dht::sign_mutable_item;
+
+        e = data;
+        std::vector<char> buf;
+        bencode(std::back_inserter(buf), e);
+        ++seq;
+        sign_mutable_item(std::pair<char const*, int>(&buf[0], buf.size())
+                          , std::pair<char const*, int>(&salt[0], salt.size())
+                          , seq, public_key.c_str(), private_key.c_str(), sig.data());
+    }
+
+    void dht_put_mutable_item(lt::session& ses, std::string private_key, std::string public_key,
+                              std::string data, std::string salt)
+    {
+        TORRENT_ASSERT(private_key.size() == 64);
+        TORRENT_ASSERT(public_key.size() == 32);
+        boost::array<char, 32> key;
+        std::copy(public_key.begin(), public_key.end(), key.begin());
+        ses.dht_put_item(key, boost::bind(&put_string, _1, _2, _3, _4
+                                          , public_key, private_key, data)
+                         , salt);
+    }
+#endif
 } // namespace unnamed
 
 
 void bind_session()
 {
 #ifndef TORRENT_DISABLE_DHT
-    void (session::*start_dht0)() = &session::start_dht;
+    void (lt::session::*dht_get_immutable_item)(sha1_hash const&) = &lt::session::dht_get_item;
+    sha1_hash (lt::session::*dht_put_immutable_item)(entry data) = &lt::session::dht_put_item;
+#endif // TORRENT_DISABLE_DHT
+
 #ifndef TORRENT_NO_DEPRECATE
-    void (session::*start_dht1)(entry const&) = &session::start_dht;
-#endif
+#ifndef TORRENT_DISABLE_DHT
+    void (lt::session::*start_dht0)() = &lt::session::start_dht;
+    void (lt::session::*start_dht1)(entry const&) = &lt::session::start_dht;
 #endif
 
     class_<session_status>("session_status")
@@ -562,7 +673,7 @@ void bind_session()
         .def_readonly("dht_global_nodes", &session_status::dht_global_nodes)
         .def_readonly("active_requests", &session_status::active_requests)
         .def_readonly("dht_total_allocations", &session_status::dht_total_allocations)
-#endif
+#endif // TORRENT_DISABLE_DHT
         .add_property("utp_stats", &get_utp_stats)
         ;
 
@@ -574,23 +685,21 @@ void bind_session()
         .def_readonly("response", &dht_lookup::responses)
         .def_readonly("branch_factor", &dht_lookup::branch_factor)
     ;
-#endif
+#endif // TORRENT_DISABLE_DHT
+#endif // TORRENT_NO_DEPRECATE
 
     enum_<storage_mode_t>("storage_mode_t")
         .value("storage_mode_allocate", storage_mode_allocate)
         .value("storage_mode_sparse", storage_mode_sparse)
-#ifndef TORRENT_NO_DEPRECATE
-        .value("storage_mode_compact", storage_mode_compact)
-#endif
     ;
 
-    enum_<session::options_t>("options_t")
-        .value("delete_files", session::delete_files)
+    enum_<lt::session::options_t>("options_t")
+        .value("delete_files", lt::session::delete_files)
     ;
 
-    enum_<session::session_flags_t>("session_flags_t")
-        .value("add_default_plugins", session::add_default_plugins)
-        .value("start_default_features", session::start_default_features)
+    enum_<lt::session::session_flags_t>("session_flags_t")
+        .value("add_default_plugins", lt::session::add_default_plugins)
+        .value("start_default_features", lt::session::start_default_features)
     ;
 
     enum_<add_torrent_params::flags_t>("add_torrent_params_flags_t")
@@ -608,8 +717,10 @@ void bind_session()
         .value("flag_sequential_download", add_torrent_params::flag_sequential_download)
         .value("flag_use_resume_save_path", add_torrent_params::flag_use_resume_save_path)
         .value("flag_merge_resume_http_seeds", add_torrent_params::flag_merge_resume_http_seeds)
+        .value("flag_stop_when_ready", add_torrent_params::flag_stop_when_ready)
     ;
     class_<cache_status>("cache_status")
+#ifndef TORRENT_NO_DEPRECATE
         .def_readonly("blocks_written", &cache_status::blocks_written)
         .def_readonly("writes", &cache_status::writes)
         .def_readonly("blocks_read", &cache_status::blocks_read)
@@ -617,57 +728,70 @@ void bind_session()
         .def_readonly("reads", &cache_status::reads)
         .def_readonly("queued_bytes", &cache_status::queued_bytes)
         .def_readonly("cache_size", &cache_status::cache_size)
+        .def_readonly("write_cache_size", &cache_status::write_cache_size)
         .def_readonly("read_cache_size", &cache_status::read_cache_size)
+        .def_readonly("pinned_blocks", &cache_status::pinned_blocks)
         .def_readonly("total_used_buffers", &cache_status::total_used_buffers)
-        .def_readonly("average_queue_time", &cache_status::average_queue_time)
         .def_readonly("average_read_time", &cache_status::average_read_time)
         .def_readonly("average_write_time", &cache_status::average_write_time)
         .def_readonly("average_hash_time", &cache_status::average_hash_time)
         .def_readonly("average_job_time", &cache_status::average_job_time)
-        .def_readonly("average_sort_time", &cache_status::average_sort_time)
-        .def_readonly("job_queue_length", &cache_status::job_queue_length)
         .def_readonly("cumulative_job_time", &cache_status::cumulative_job_time)
         .def_readonly("cumulative_read_time", &cache_status::cumulative_read_time)
         .def_readonly("cumulative_write_time", &cache_status::cumulative_write_time)
         .def_readonly("cumulative_hash_time", &cache_status::cumulative_hash_time)
-        .def_readonly("cumulative_sort_time", &cache_status::cumulative_sort_time)
         .def_readonly("total_read_back", &cache_status::total_read_back)
         .def_readonly("read_queue_size", &cache_status::read_queue_size)
+        .def_readonly("blocked_jobs", &cache_status::blocked_jobs)
+        .def_readonly("queued_jobs", &cache_status::queued_jobs)
+        .def_readonly("peak_queued", &cache_status::peak_queued)
+        .def_readonly("pending_jobs", &cache_status::pending_jobs)
+        .def_readonly("num_jobs", &cache_status::num_jobs)
+        .def_readonly("num_read_jobs", &cache_status::num_read_jobs)
+        .def_readonly("num_write_jobs", &cache_status::num_write_jobs)
+        .def_readonly("arc_mru_size", &cache_status::arc_mru_size)
+        .def_readonly("arc_mru_ghost_size", &cache_status::arc_mru_ghost_size)
+        .def_readonly("arc_mfu_size", &cache_status::arc_mfu_size)
+        .def_readonly("arc_mfu_ghost_size", &cache_status::arc_mfu_ghost_size)
+#endif
     ;
 
-    class_<session, boost::noncopyable>("session", no_init)
+    class_<lt::session, boost::noncopyable>("session", no_init)
+        .def("__init__", boost::python::make_constructor(&make_session
+                , default_call_policies()
+                , (arg("settings")
+                , arg("flags")=lt::session::start_default_features
+                    | lt::session::add_default_plugins))
+        )
+#ifndef TORRENT_NO_DEPRECATE
         .def(
-            init<fingerprint, int>((
+            init<fingerprint, int, boost::uint32_t>((
                 arg("fingerprint")=fingerprint("LT",0,1,0,0)
-                , arg("flags")=session::start_default_features | session::add_default_plugins))
-        )
-        .def("post_torrent_updates", allow_threads(&session::post_torrent_updates))
-        .def(
-            "listen_on", &listen_on
-          , (arg("min"), "max", arg("interface") = (char const*)0, arg("flags") = 0)
+                , arg("flags")=lt::session::start_default_features | lt::session::add_default_plugins
+                , arg("alert_mask")=int(alert::error_notification)))
         )
+#endif
+        .def("post_torrent_updates", allow_threads(&lt::session::post_torrent_updates))
+        .def("post_session_stats", allow_threads(&lt::session::post_session_stats))
         .def("outgoing_ports", &outgoing_ports)
-        .def("is_listening", allow_threads(&session::is_listening))
-        .def("listen_port", allow_threads(&session::listen_port))
-        .def("status", allow_threads(&session::status))
+        .def("is_listening", allow_threads(&lt::session::is_listening))
+        .def("listen_port", allow_threads(&lt::session::listen_port))
 #ifndef TORRENT_DISABLE_DHT
-        .def("add_dht_node", add_dht_node)
+        .def("add_dht_node", &add_dht_node)
         .def(
             "add_dht_router", &add_dht_router
           , (arg("router"), "port")
         )
-        .def("is_dht_running", allow_threads(&session::is_dht_running))
-        .def("set_dht_settings", allow_threads(&session::set_dht_settings))
-        .def("get_dht_settings", allow_threads(&session::get_dht_settings))
-        .def("start_dht", allow_threads(start_dht0))
-        .def("stop_dht", allow_threads(&session::stop_dht))
-#ifndef TORRENT_NO_DEPRECATE
-        .def("start_dht", allow_threads(start_dht1))
-        .def("dht_state", allow_threads(&session::dht_state))
-        .def("set_dht_proxy", allow_threads(&session::set_dht_proxy))
-        .def("dht_proxy", allow_threads(&session::dht_proxy))
-#endif
-#endif
+        .def("is_dht_running", allow_threads(&lt::session::is_dht_running))
+        .def("set_dht_settings", allow_threads(&lt::session::set_dht_settings))
+        .def("get_dht_settings", allow_threads(&lt::session::get_dht_settings))
+        .def("dht_get_immutable_item", allow_threads(dht_get_immutable_item))
+        .def("dht_get_mutable_item", &dht_get_mutable_item)
+        .def("dht_put_immutable_item", allow_threads(dht_put_immutable_item))
+        .def("dht_put_mutable_item", &dht_put_mutable_item)
+        .def("dht_get_peers", allow_threads(&lt::session::dht_get_peers))
+        .def("dht_announce", allow_threads(&lt::session::dht_announce))
+#endif // TORRENT_DISABLE_DHT
         .def("add_torrent", &add_torrent)
         .def("async_add_torrent", &async_add_torrent)
 #ifndef BOOST_NO_EXCEPTIONS
@@ -680,116 +804,127 @@ void bind_session()
                 arg("paused") = false
             )
         )
-#endif
-#endif
+#endif // TORRENT_NO_DEPRECATE
+#endif // BOOST_NO_EXCEPTIONS
+        .def("remove_torrent", allow_threads(&lt::session::remove_torrent), arg("option") = 0)
+#ifndef TORRENT_NO_DEPRECATE
         .def("add_feed", &add_feed)
-        .def("remove_torrent", allow_threads(&session::remove_torrent), arg("option") = 0)
-#ifndef TORRENT_NO_DEPRECATE
-        .def("set_local_download_rate_limit", allow_threads(&session::set_local_download_rate_limit))
-        .def("local_download_rate_limit", allow_threads(&session::local_download_rate_limit))
-
-        .def("set_local_upload_rate_limit", allow_threads(&session::set_local_upload_rate_limit))
-        .def("local_upload_rate_limit", allow_threads(&session::local_upload_rate_limit))
-
-        .def("set_download_rate_limit", allow_threads(&session::set_download_rate_limit))
-        .def("download_rate_limit", allow_threads(&session::download_rate_limit))
-
-        .def("set_upload_rate_limit", allow_threads(&session::set_upload_rate_limit))
-        .def("upload_rate_limit", allow_threads(&session::upload_rate_limit))
-
-        .def("set_max_uploads", allow_threads(&session::set_max_uploads))
-        .def("set_max_connections", allow_threads(&session::set_max_connections))
-        .def("max_connections", allow_threads(&session::max_connections))
-        .def("set_max_half_open_connections", allow_threads(&session::set_max_half_open_connections))
-        .def("num_connections", allow_threads(&session::num_connections))
-        .def("set_settings", &session::set_settings)
-        .def("settings", &session::settings)
-        .def("get_settings", &session_get_settings)
-#else
+        .def("status", allow_threads(&lt::session::status))
         .def("settings", &session_get_settings)
-        .def("get_settings", &session_get_settings)
-#endif
         .def("set_settings", &session_set_settings)
-#ifndef TORRENT_DISABLE_ENCRYPTION
-        .def("set_pe_settings", allow_threads(&session::set_pe_settings))
-        .def("get_pe_settings", allow_threads(&session::get_pe_settings))
 #endif
-#ifndef TORRENT_DISABLE_GEO_IP
-        .def("load_asnum_db", &load_asnum_db)
-        .def("load_country_db", &load_country_db)
-#endif
-        .def("load_state", &load_state)
-        .def("save_state", &save_state, (arg("entry"), arg("flags") = 0xffffffff))
+        .def("get_settings", &session_get_settings)
+        .def("apply_settings", &session_apply_settings)
 #ifndef TORRENT_NO_DEPRECATE
-        .def("set_severity_level", allow_threads(&session::set_severity_level))
-        .def("set_alert_queue_size_limit", allow_threads(&session::set_alert_queue_size_limit))
+#ifndef TORRENT_DISABLE_ENCRYPTION
+        .def("set_pe_settings", allow_threads(&lt::session::set_pe_settings))
+        .def("get_pe_settings", allow_threads(&lt::session::get_pe_settings))
 #endif
-        .def("set_alert_mask", allow_threads(&session::set_alert_mask))
-        .def("pop_alert", &pop_alert)
+#endif
+        .def("load_state", &load_state, (arg("entry"), arg("flags") = 0xffffffff))
+        .def("save_state", &save_state, (arg("entry"), arg("flags") = 0xffffffff))
         .def("pop_alerts", &pop_alerts)
-        .def("wait_for_alert", &wait_for_alert, return_internal_reference<>())
+        .def("wait_for_alert", &wait_for_alert
+#ifdef TORRENT_NO_DEPRECATE
+            , return_internal_reference<>()
+#endif
+        )
         .def("add_extension", &add_extension)
 #ifndef TORRENT_NO_DEPRECATE
-        .def("set_peer_proxy", allow_threads(&session::set_peer_proxy))
-        .def("set_tracker_proxy", allow_threads(&session::set_tracker_proxy))
-        .def("set_web_seed_proxy", allow_threads(&session::set_web_seed_proxy))
-        .def("peer_proxy", allow_threads(&session::peer_proxy))
-        .def("tracker_proxy", allow_threads(&session::tracker_proxy))
-        .def("web_seed_proxy", allow_threads(&session::web_seed_proxy))
-#endif
+        .def("pop_alert", &pop_alert)
 #if TORRENT_USE_I2P
-        .def("set_i2p_proxy", allow_threads(&session::set_i2p_proxy))
-        .def("i2p_proxy", allow_threads(&session::i2p_proxy))
+        .def("set_i2p_proxy", allow_threads(&lt::session::set_i2p_proxy))
+        .def("i2p_proxy", allow_threads(&lt::session::i2p_proxy))
 #endif
-        .def("set_proxy", allow_threads(&session::set_proxy))
-        .def("proxy", allow_threads(&session::proxy))
-        .def("start_upnp", &start_upnp)
-        .def("stop_upnp", allow_threads(&session::stop_upnp))
-        .def("start_lsd", allow_threads(&session::start_lsd))
-        .def("stop_lsd", allow_threads(&session::stop_lsd))
-        .def("start_natpmp", &start_natpmp)
-        .def("stop_natpmp", allow_threads(&session::stop_natpmp))
-        .def("set_ip_filter", allow_threads(&session::set_ip_filter))
-        .def("get_ip_filter", allow_threads(&session::get_ip_filter))
-        .def("find_torrent", allow_threads(&session::find_torrent))
+#endif
+        .def("set_ip_filter", allow_threads(&lt::session::set_ip_filter))
+        .def("get_ip_filter", allow_threads(&lt::session::get_ip_filter))
+        .def("find_torrent", allow_threads(&lt::session::find_torrent))
         .def("get_torrents", &get_torrents)
-        .def("pause", allow_threads(&session::pause))
-        .def("resume", allow_threads(&session::resume))
-        .def("is_paused", allow_threads(&session::is_paused))
-        .def("id", allow_threads(&session::id))
-        .def("get_cache_status", allow_threads(&session::get_cache_status))
-        .def("get_cache_info", get_cache_info)
-        .def("set_peer_id", allow_threads(&session::set_peer_id))
-        .def("add_port_mapping", allow_threads(&session::add_port_mapping))
-        .def("delete_port_mapping", allow_threads(&session::delete_port_mapping))
+        .def("pause", allow_threads(&lt::session::pause))
+        .def("resume", allow_threads(&lt::session::resume))
+        .def("is_paused", allow_threads(&lt::session::is_paused))
+        .def("id", allow_threads(&lt::session::id))
+        .def("get_cache_info", &get_cache_info1, (arg("handle") = torrent_handle(), arg("flags") = 0))
+        .def("add_port_mapping", allow_threads(&lt::session::add_port_mapping))
+        .def("delete_port_mapping", allow_threads(&lt::session::delete_port_mapping))
+
+#ifndef TORRENT_NO_DEPRECATE
+        .def(
+            "listen_on", &listen_on
+          , (arg("min"), "max", arg("interface") = (char const*)0, arg("flags") = 0)
+        )
+#ifndef TORRENT_DISABLE_DHT
+        .def("start_dht", allow_threads(start_dht0))
+        .def("stop_dht", allow_threads(&lt::session::stop_dht))
+        .def("start_dht", allow_threads(start_dht1))
+        .def("dht_state", allow_threads(&lt::session::dht_state))
+        .def("set_dht_proxy", allow_threads(&lt::session::set_dht_proxy))
+        .def("dht_proxy", allow_threads(&lt::session::dht_proxy))
+#endif
+        .def("set_local_download_rate_limit", allow_threads(&lt::session::set_local_download_rate_limit))
+        .def("local_download_rate_limit", allow_threads(&lt::session::local_download_rate_limit))
+        .def("set_local_upload_rate_limit", allow_threads(&lt::session::set_local_upload_rate_limit))
+        .def("local_upload_rate_limit", allow_threads(&lt::session::local_upload_rate_limit))
+        .def("set_download_rate_limit", allow_threads(&lt::session::set_download_rate_limit))
+        .def("download_rate_limit", allow_threads(&lt::session::download_rate_limit))
+        .def("set_upload_rate_limit", allow_threads(&lt::session::set_upload_rate_limit))
+        .def("upload_rate_limit", allow_threads(&lt::session::upload_rate_limit))
+        .def("set_max_uploads", allow_threads(&lt::session::set_max_uploads))
+        .def("set_max_connections", allow_threads(&lt::session::set_max_connections))
+        .def("max_connections", allow_threads(&lt::session::max_connections))
+        .def("num_connections", allow_threads(&lt::session::num_connections))
+        .def("load_asnum_db", &load_asnum_db)
+        .def("load_country_db", &load_country_db)
+        .def("set_max_half_open_connections", allow_threads(&lt::session::set_max_half_open_connections))
+        .def("set_severity_level", allow_threads(&lt::session::set_severity_level))
+        .def("set_alert_queue_size_limit", allow_threads(&lt::session::set_alert_queue_size_limit))
+        .def("set_alert_mask", allow_threads(&lt::session::set_alert_mask))
+        .def("set_peer_proxy", allow_threads(&lt::session::set_peer_proxy))
+        .def("set_tracker_proxy", allow_threads(&lt::session::set_tracker_proxy))
+        .def("set_web_seed_proxy", allow_threads(&lt::session::set_web_seed_proxy))
+        .def("peer_proxy", allow_threads(&lt::session::peer_proxy))
+        .def("tracker_proxy", allow_threads(&lt::session::tracker_proxy))
+        .def("web_seed_proxy", allow_threads(&lt::session::web_seed_proxy))
+        .def("set_proxy", allow_threads(&lt::session::set_proxy))
+        .def("proxy", allow_threads(&lt::session::proxy))
+        .def("start_upnp", &start_upnp)
+        .def("stop_upnp", allow_threads(&lt::session::stop_upnp))
+        .def("start_lsd", allow_threads(&lt::session::start_lsd))
+        .def("stop_lsd", allow_threads(&lt::session::stop_lsd))
+        .def("start_natpmp", &start_natpmp)
+        .def("stop_natpmp", allow_threads(&lt::session::stop_natpmp))
+        .def("get_cache_status", &get_cache_status)
+        .def("get_cache_info", &get_cache_info2)
+        .def("set_peer_id", allow_threads(&lt::session::set_peer_id))
+#endif // TORRENT_NO_DEPRECATE
         ;
 
-    enum_<session::protocol_type>("protocol_type")
-        .value("udp", session::udp)
-        .value("tcp", session::tcp)
+    enum_<lt::session::protocol_type>("protocol_type")
+        .value("udp", lt::session::udp)
+        .value("tcp", lt::session::tcp)
     ;
 
-    enum_<session::save_state_flags_t>("save_state_flags_t")
-        .value("save_settings", session::save_settings)
-        .value("save_dht_settings", session::save_dht_settings)
-        .value("save_dht_state", session::save_dht_state)
-        .value("save_i2p_proxy", session::save_i2p_proxy)
-        .value("save_encryption_settings", session:: save_encryption_settings)
-        .value("save_as_map", session::save_as_map)
-        .value("save_proxy", session::save_proxy)
+    enum_<lt::session::save_state_flags_t>("save_state_flags_t")
+        .value("save_settings", lt::session::save_settings)
+        .value("save_dht_settings", lt::session::save_dht_settings)
+        .value("save_dht_state", lt::session::save_dht_state)
+        .value("save_encryption_settings", lt::session:: save_encryption_settings)
 #ifndef TORRENT_NO_DEPRECATE
-        .value("save_dht_proxy", session::save_dht_proxy)
-        .value("save_peer_proxy", session::save_peer_proxy)
-        .value("save_web_proxy", session::save_web_proxy)
-        .value("save_tracker_proxy", session::save_tracker_proxy)
+        .value("save_as_map", lt::session::save_as_map)
+        .value("save_i2p_proxy", lt::session::save_i2p_proxy)
+        .value("save_proxy", lt::session::save_proxy)
+        .value("save_dht_proxy", lt::session::save_dht_proxy)
+        .value("save_peer_proxy", lt::session::save_peer_proxy)
+        .value("save_web_proxy", lt::session::save_web_proxy)
+        .value("save_tracker_proxy", lt::session::save_tracker_proxy)
 #endif
     ;
 
-    enum_<session::listen_on_flags_t>("listen_on_flags_t")
 #ifndef TORRENT_NO_DEPRECATE
-        .value("listen_reuse_address", session::listen_reuse_address)
-#endif
-        .value("listen_no_system_port", session::listen_no_system_port)
+    enum_<lt::session::listen_on_flags_t>("listen_on_flags_t")
+        .value("listen_reuse_address", lt::session::listen_reuse_address)
+        .value("listen_no_system_port", lt::session::listen_no_system_port)
     ;
 
     class_<feed_handle>("feed_handle")
@@ -798,14 +933,26 @@ void bind_session()
         .def("set_settings", &set_feed_settings)
         .def("settings", &get_feed_settings)
     ;
+#endif
 
-    register_ptr_to_python<std::auto_ptr<alert> >();
+    typedef void (*mem_preset2)(settings_pack& s);
+    typedef void (*perf_preset2)(settings_pack& s);
 
-    def("high_performance_seed", high_performance_seed);
-    def("min_memory_usage", min_memory_usage);
+#ifndef TORRENT_NO_DEPRECATE
 
+    typedef session_settings (*mem_preset1)();
+    typedef session_settings (*perf_preset1)();
+
+    def("high_performance_seed", (perf_preset1)high_performance_seed);
+    def("min_memory_usage", (mem_preset1)min_memory_usage);
     scope().attr("create_metadata_plugin") = "metadata_transfer";
+#endif
+
+    def("high_performance_seed", (perf_preset2)high_performance_seed);
+    def("min_memory_usage", (mem_preset2)min_memory_usage);
+
     scope().attr("create_ut_metadata_plugin") = "ut_metadata";
     scope().attr("create_ut_pex_plugin") = "ut_pex";
     scope().attr("create_smart_ban_plugin") = "smart_ban";
 }
+
