@@ -2,10 +2,15 @@
 // subject to the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/python.hpp>
+#include "boost_python.hpp"
 #include <boost/python/tuple.hpp>
+#include <boost/python/stl_iterator.hpp>
 #include <libtorrent/torrent_handle.hpp>
+#include <libtorrent/torrent_info.hpp>
+#include <libtorrent/torrent_status.hpp>
+#include <libtorrent/entry.hpp>
 #include <libtorrent/peer_info.hpp>
+#include "libtorrent/announce_entry.hpp"
 #include <libtorrent/storage.hpp>
 #include <boost/lexical_cast.hpp>
 #include "gil.hpp"
@@ -80,11 +85,11 @@ namespace
 
 list file_progress(torrent_handle& handle, int flags)
 {
-    std::vector<size_type> p;
+    std::vector<boost::int64_t> p;
 
     {
         allow_threading_guard guard;
-        boost::intrusive_ptr<const torrent_info> ti = handle.torrent_file();
+        boost::shared_ptr<const torrent_info> ti = handle.torrent_file();
         if (ti)
         {
            p.reserve(ti->num_files());
@@ -94,7 +99,7 @@ list file_progress(torrent_handle& handle, int flags)
 
     list result;
 
-    for (std::vector<size_type>::iterator i(p.begin()), e(p.end()); i != e; ++i)
+    for (std::vector<boost::int64_t>::iterator i(p.begin()), e(p.end()); i != e; ++i)
         result.append(*i);
 
     return result;
@@ -117,44 +122,44 @@ list get_peer_info(torrent_handle const& handle)
     return result;
 }
 
+namespace
+{
+   template <typename T>
+   T extract_fn(object o)
+   {
+      return boost::python::extract<T>(o);
+   }
+}
+
 void prioritize_pieces(torrent_handle& info, object o)
 {
-   std::vector<int> result;
-   try
+   stl_input_iterator<object> begin(o), end;
+   if (begin == end) return;
+
+   // determine which overload should be selected. the one taking a list of
+   // priorities or the one taking a list of piece -> priority mappings
+   bool const is_piece_list = extract<std::pair<int, int> >(*begin).check();
+
+   if (is_piece_list)
    {
-      object iter_obj = object( handle<>( PyObject_GetIter( o.ptr() ) ));
-      while( 1 )
-      {
-         object obj = extract<object>( iter_obj.attr( "next" )() );
-         result.push_back(extract<int const>( obj ));
-      }
+      std::vector<std::pair<int, int> > piece_list;
+      std::transform(begin, end, std::back_inserter(piece_list)
+         , &extract_fn<std::pair<int, int> >);
+      info.prioritize_pieces(piece_list);
    }
-   catch( error_already_set )
+   else
    {
-      PyErr_Clear();
-      info.prioritize_pieces(result);
-      return;
+      std::vector<int> priority_vector;
+      std::transform(begin, end, std::back_inserter(priority_vector)
+         , &extract_fn<int>);
+      info.prioritize_pieces(priority_vector);
    }
 }
 
 void prioritize_files(torrent_handle& info, object o)
 {
-   std::vector<int> result;
-   try
-   {
-      object iter_obj = object( handle<>( PyObject_GetIter( o.ptr() ) ));
-      while( 1 )
-      {
-         object obj = extract<object>( iter_obj.attr( "next" )() );
-         result.push_back(extract<int const>( obj ));
-      }
-   }
-   catch( error_already_set )
-   {
-      PyErr_Clear();
-      info.prioritize_files(result);
-      return;
-   }
+   stl_input_iterator<int const> begin(o), end;
+   info.prioritize_files(std::vector<int> (begin, end));
 }
 
 list file_priorities(torrent_handle& handle)
@@ -312,19 +317,21 @@ void connect_peer(torrent_handle& th, tuple ip, int source)
 #ifndef TORRENT_NO_DEPRECATE
 #if BOOST_VERSION > 104200
 
-boost::intrusive_ptr<const torrent_info> get_torrent_info(torrent_handle const& h)
+boost::shared_ptr<const torrent_info> get_torrent_info(torrent_handle const& h)
 {
-	return boost::intrusive_ptr<const torrent_info>(&h.get_torrent_info());
+	allow_threading_guard guard;
+	return h.torrent_file();
 }
 
 #else
 
-boost::intrusive_ptr<torrent_info> get_torrent_info(torrent_handle const& h)
+boost::shared_ptr<torrent_info> get_torrent_info(torrent_handle const& h)
 {
-	// I can't figure out how to expose intrusive_ptr<const torrent_info>
+	// I can't figure out how to expose shared_ptr<const torrent_info>
 	// as well as supporting mutable instances. So, this hack is better
 	// than compilation errors. It seems to work on newer versions of boost though
-   return boost::intrusive_ptr<torrent_info>(const_cast<torrent_info*>(&h.get_torrent_info()));
+	allow_threading_guard guard;
+	return boost::const_pointer_cast<torrent_info>(h.torrent_file());
 }
 
 #endif
@@ -338,7 +345,8 @@ void set_peer_download_limit(torrent_handle& th, tuple const& ip, int limit)
 {
     th.set_peer_download_limit(tuple_to_endpoint(ip), limit);
 }
-#endif
+
+#endif // TORRENT_NO_DEPRECAE
 
 void add_piece(torrent_handle& th, int piece, char const *data, int flags)
 {
@@ -366,10 +374,13 @@ void bind_torrent_handle()
     void (torrent_handle::*rename_file1)(int, std::wstring const&) const = &torrent_handle::rename_file;
 #endif
 
+#ifndef TORRENT_NO_DEPRECATE
+    // deprecated in 1.1
 #ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES
     bool (torrent_handle::*resolve_countries0)() const = &torrent_handle::resolve_countries;
     void (torrent_handle::*resolve_countries1)(bool) = &torrent_handle::resolve_countries;
 #endif
+#endif // TORRENT_NO_DEPRECATE
 
 #define _ allow_threads
 
@@ -395,6 +406,7 @@ void bind_torrent_handle()
         .def("is_valid", _(&torrent_handle::is_valid))
         .def("pause", _(&torrent_handle::pause), arg("flags") = 0)
         .def("resume", _(&torrent_handle::resume))
+        .def("stop_when_ready", _(&torrent_handle::stop_when_ready))
         .def("clear_error", _(&torrent_handle::clear_error))
         .def("set_priority", _(&torrent_handle::set_priority))
         .def("super_seeding", super_seeding1)
@@ -406,13 +418,14 @@ void bind_torrent_handle()
         .def("queue_position_top", _(&torrent_handle::queue_position_top))
         .def("queue_position_bottom", _(&torrent_handle::queue_position_bottom))
 
+        // deprecated
+#ifndef TORRENT_NO_DEPRECATE
+        // resolve countries deprecated in 1.1
 #ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES
         .def("resolve_countries", _(resolve_countries0))
         .def("resolve_countries", _(resolve_countries1))
 #endif
-        // deprecated
-#ifndef TORRENT_NO_DEPRECATE
-        .def("get_torrent_info", get_torrent_info)
+        .def("get_torrent_info", &get_torrent_info)
         .def("super_seeding", super_seeding0)
         .def("filter_piece", _(&torrent_handle::filter_piece))
         .def("is_piece_filtered", _(&torrent_handle::is_piece_filtered))
@@ -443,7 +456,7 @@ void bind_torrent_handle()
         .def("save_resume_data", _(&torrent_handle::save_resume_data), arg("flags") = 0)
         .def("need_save_resume_data", _(&torrent_handle::need_save_resume_data))
         .def("force_reannounce", _(force_reannounce0)
-			  , (arg("seconds") = 0, arg("tracker_idx") = -1))
+            , (arg("seconds") = 0, arg("tracker_idx") = -1))
 #ifndef TORRENT_DISABLE_DHT
         .def("force_dht_announce", _(&torrent_handle::force_dht_announce))
 #endif
@@ -468,7 +481,9 @@ void bind_torrent_handle()
         .def("max_uploads", _(&torrent_handle::max_uploads))
         .def("set_max_connections", _(&torrent_handle::set_max_connections))
         .def("max_connections", _(&torrent_handle::max_connections))
+#ifndef TORRENT_NO_DEPRECATE
         .def("set_tracker_login", _(&torrent_handle::set_tracker_login))
+#endif
         .def("move_storage", _(move_storage0), (arg("path"), arg("flags") = 0))
         .def("info_hash", _(&torrent_handle::info_hash))
         .def("force_recheck", _(&torrent_handle::force_recheck))
