@@ -2,11 +2,18 @@
 // subject to the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/python.hpp>
-#include <libtorrent/torrent_info.hpp>
-#include "libtorrent/intrusive_ptr_base.hpp"
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
+#include "boost_python.hpp"
+#include <boost/shared_ptr.hpp>
+
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
+#include "libtorrent/torrent_info.hpp"
 #include "libtorrent/session_settings.hpp"
-#include "libtorrent/bencode.hpp"
+#include "libtorrent/time.hpp"
+#include "libtorrent/socket_io.hpp"
+#include "libtorrent/announce_entry.hpp"
 #include "bytes.hpp"
 
 using namespace boost::python;
@@ -38,7 +45,7 @@ namespace
 
         for (list_type::const_iterator i = ti.nodes().begin(); i != ti.nodes().end(); ++i)
         {
-            result.append(make_tuple(i->first, i->second));
+            result.append(boost::python::make_tuple(i->first, i->second));
         }
 
         return result;
@@ -56,11 +63,6 @@ namespace
             d["type"] = i->type;
             d["auth"] = i->auth;
             d["extra_headers"] = i->extra_headers;
-            d["retry"] = total_seconds(i->retry - min_time());
-            d["resolving"] = i->resolving;
-            d["removed"] = i->removed;
-            d["endpoint"] = make_tuple(
-                boost::lexical_cast<std::string>(i->endpoint.address()), i->endpoint.port());
             ret.append(d);
         }
 
@@ -88,57 +90,17 @@ namespace
         ti.set_merkle_tree(h);
     }
 
-#ifndef TORRENT_NO_DEPRECATE
-    file_storage::iterator begin_files(torrent_info& i)
+    bytes hash_for_piece(torrent_info const& ti, int i)
     {
-        return i.begin_files();
+        return bytes(ti.hash_for_piece(i).to_string());
     }
 
-    file_storage::iterator end_files(torrent_info& i)
+    bytes metadata(torrent_info const& ti)
     {
-        return i.end_files();
-    }
-#endif
-
-    void remap_files(torrent_info& ti, list files) {
-        file_storage st;
-        for (int i = 0, e = len(files); i < e; ++i)
-            st.add_file(extract<file_entry>(files[i]));
-
-        ti.remap_files(st);
+        return bytes(ti.metadata().get(), ti.metadata_size());
     }
 
-    list files(torrent_info const& ti, bool storage) {
-        list result;
-
-        for (int i = 0; i < ti.num_files(); ++i)
-            result.append(ti.files().at(i));
-
-        return result;
-    }
-
-    list orig_files(torrent_info const& ti, bool storage) {
-        list result;
-
-        file_storage const& st = ti.orig_files();
-
-        for (int i = 0; i < st.num_files(); ++i)
-            result.append(st.at(i));
-
-        return result;
-    }
-
-    std::string hash_for_piece(torrent_info const& ti, int i)
-    {
-        return ti.hash_for_piece(i).to_string();
-    }
-
-    std::string metadata(torrent_info const& ti) {
-        std::string result(ti.metadata().get(), ti.metadata_size());
-        return result;
-    }
-
-    list map_block(torrent_info& ti, int piece, size_type offset, int size)
+    list map_block(torrent_info& ti, int piece, boost::int64_t offset, int size)
     {
        std::vector<file_slice> p = ti.map_block(piece, offset, size);
        list result;
@@ -161,69 +123,74 @@ namespace
     bool get_complete_sent(announce_entry const& ae) { return ae.complete_sent; }
     bool get_send_stats(announce_entry const& ae) { return ae.send_stats; }
 
-
-    size_type get_size(file_entry const& fe) { return fe.size; }
-    size_type get_offset(file_entry const& fe) { return fe.offset; }
-    size_type get_file_base(file_entry const& fe) { return fe.file_base; }
+#if !defined TORRENT_NO_DEPRECATE
+    boost::int64_t get_size(file_entry const& fe) { return fe.size; }
+    boost::int64_t get_offset(file_entry const& fe) { return fe.offset; }
+    boost::int64_t get_file_base(file_entry const& fe) { return fe.file_base; }
     void set_file_base(file_entry& fe, int b) { fe.file_base = b; }
     bool get_pad_file(file_entry const& fe) { return fe.pad_file; }
     bool get_executable_attribute(file_entry const& fe) { return fe.executable_attribute; }
     bool get_hidden_attribute(file_entry const& fe) { return fe.hidden_attribute; }
     bool get_symlink_attribute(file_entry const& fe) { return fe.symlink_attribute; }
+#endif
 
 } // namespace unnamed
 
-boost::intrusive_ptr<torrent_info> buffer_constructor0(char const* buf, int len, int flags)
+boost::shared_ptr<torrent_info> buffer_constructor0(char const* buf, int len, int flags)
 {
    error_code ec;
-   boost::intrusive_ptr<torrent_info> ret(new torrent_info(buf, len, ec, flags));
+   boost::shared_ptr<torrent_info> ret(boost::make_shared<torrent_info>(buf
+		, len, boost::ref(ec), flags));
 #ifndef BOOST_NO_EXCEPTIONS
    if (ec) throw libtorrent_exception(ec);
 #endif
    return ret;
 }
 
-boost::intrusive_ptr<torrent_info> buffer_constructor1(char const* buf, int len)
+boost::shared_ptr<torrent_info> buffer_constructor1(char const* buf, int len)
 {
 	return buffer_constructor0(buf, len, 0);
 }
 
-boost::intrusive_ptr<torrent_info> file_constructor0(std::string const& filename, int flags)
+boost::shared_ptr<torrent_info> file_constructor0(std::string const& filename, int flags)
 {
    error_code ec;
-   boost::intrusive_ptr<torrent_info> ret(new torrent_info(filename, ec, flags));
+   boost::shared_ptr<torrent_info> ret(boost::make_shared<torrent_info>(filename
+		, boost::ref(ec), flags));
 #ifndef BOOST_NO_EXCEPTIONS
    if (ec) throw libtorrent_exception(ec);
 #endif
    return ret;
 }
 
-boost::intrusive_ptr<torrent_info> file_constructor1(std::string const& filename)
+boost::shared_ptr<torrent_info> file_constructor1(std::string const& filename)
 {
 	return file_constructor0(filename, 0);
 }
 
-boost::intrusive_ptr<torrent_info> bencoded_constructor0(entry const& ent, int flags)
+boost::shared_ptr<torrent_info> bencoded_constructor0(entry const& ent, int flags)
 {
-	error_code ec;
-	lazy_entry e;
 	std::vector<char> buf;
 	bencode(std::back_inserter(buf), ent);
-	if (buf.size() == 0 || lazy_bdecode(&buf[0], &buf[0] + buf.size(), e, ec) != 0)
+
+	bdecode_node e;
+	error_code ec;
+	if (buf.size() == 0 || bdecode(&buf[0], &buf[0] + buf.size(), e, ec) != 0)
 	{
 #ifndef BOOST_NO_EXCEPTIONS
 		throw invalid_torrent_file(ec);
 #endif
 	}
 
-	boost::intrusive_ptr<torrent_info> ret(new torrent_info(e, ec, flags));
+	boost::shared_ptr<torrent_info> ret(boost::make_shared<torrent_info>(e
+			, boost::ref(ec), flags));
 #ifndef BOOST_NO_EXCEPTIONS
 	if (ec) throw libtorrent_exception(ec);
 #endif
 	return ret;
 }
 
-boost::intrusive_ptr<torrent_info> bencoded_constructor1(entry const& ent)
+boost::shared_ptr<torrent_info> bencoded_constructor1(entry const& ent)
 {
 	return bencoded_constructor0(ent, 0);
 }
@@ -243,21 +210,20 @@ void bind_torrent_info()
         .def_readwrite("size", &file_slice::size)
         ;
 
-    class_<torrent_info, boost::intrusive_ptr<torrent_info> >("torrent_info", no_init)
+    class_<torrent_info, boost::shared_ptr<torrent_info> >("torrent_info", no_init)
         .def(init<sha1_hash const&, int>((arg("info_hash"), arg("flags") = 0)))
+        .def("__init__", make_constructor(&bencoded_constructor0))
+        .def("__init__", make_constructor(&bencoded_constructor1))
         .def("__init__", make_constructor(&buffer_constructor0))
         .def("__init__", make_constructor(&buffer_constructor1))
         .def("__init__", make_constructor(&file_constructor0))
         .def("__init__", make_constructor(&file_constructor1))
-        .def("__init__", make_constructor(&bencoded_constructor0))
-        .def("__init__", make_constructor(&bencoded_constructor1))
-        .def(init<torrent_info const&, int>((arg("ti"), arg("flags") = 0)))
+        .def(init<torrent_info const&>((arg("ti"))))
 
 #if TORRENT_USE_WSTRING && !defined TORRENT_NO_DEPRECATE
         .def(init<std::wstring, int>((arg("file"), arg("flags") = 0)))
 #endif
 
-        .def("remap_files", &remap_files)
         .def("add_tracker", &torrent_info::add_tracker, arg("url"))
         .def("add_url_seed", &torrent_info::add_url_seed)
         .def("add_http_seed", &torrent_info::add_http_seed)
@@ -270,22 +236,23 @@ void bind_torrent_info()
         .def("piece_length", &torrent_info::piece_length)
         .def("num_pieces", &torrent_info::num_pieces)
         .def("info_hash", &torrent_info::info_hash, copy)
-#ifndef TORRENT_NO_DEPRECATE
-        .def("file_at_offset", &torrent_info::file_at_offset)
-#endif
         .def("hash_for_piece", &hash_for_piece)
         .def("merkle_tree", get_merkle_tree)
         .def("set_merkle_tree", set_merkle_tree)
         .def("piece_size", &torrent_info::piece_size)
 
-        .def("num_files", &torrent_info::num_files, (arg("storage")=false))
-        .def("file_at", &torrent_info::file_at)
-        .def("files", &files, (arg("storage")=false))
-        .def("orig_files", &orig_files, (arg("storage")=false))
+        .def("num_files", &torrent_info::num_files)
         .def("rename_file", rename_file0)
-#if TORRENT_USE_WSTRING && !defined TORRENT_NO_DEPRECATE
+        .def("remap_files", &torrent_info::remap_files)
+        .def("files", &torrent_info::files, return_internal_reference<>())
+        .def("orig_files", &torrent_info::orig_files, return_internal_reference<>())
+#if !defined TORRENT_NO_DEPRECATE
+        .def("file_at", &torrent_info::file_at)
+        .def("file_at_offset", &torrent_info::file_at_offset)
+#if TORRENT_USE_WSTRING
         .def("rename_file", rename_file1)
-#endif
+#endif // TORRENT_USE_WSTRING
+#endif // TORRENT_NO_DEPRECATE
 
         .def("priv", &torrent_info::priv)
         .def("trackers", range(begin_trackers, end_trackers))
@@ -300,6 +267,7 @@ void bind_torrent_info()
         .def("map_file", &torrent_info::map_file)
         ;
 
+#if !defined TORRENT_NO_DEPRECATE
     class_<file_entry>("file_entry")
         .def_readwrite("path", &file_entry::path)
         .def_readwrite("symlink_path", &file_entry::symlink_path)
@@ -313,6 +281,7 @@ void bind_torrent_info()
         .add_property("size", &get_size)
         .add_property("file_base", &get_file_base, &set_file_base)
         ;
+#endif
 
     class_<announce_entry>("announce_entry", init<std::string const&>())
         .def_readwrite("url", &announce_entry::url)
@@ -327,7 +296,6 @@ void bind_torrent_info()
         .add_property("send_stats", &get_send_stats)
 
         .def("reset", &announce_entry::reset)
-        .def("failed", &announce_entry::failed, arg("retry_interval") = 0)
         .def("can_announce", &announce_entry::can_announce)
         .def("is_working", &announce_entry::is_working)
         .def("trim", &announce_entry::trim)
@@ -341,8 +309,8 @@ void bind_torrent_info()
     ;
 
 #if BOOST_VERSION > 104200
-    implicitly_convertible<boost::intrusive_ptr<torrent_info>, boost::intrusive_ptr<const torrent_info> >();
-    boost::python::register_ptr_to_python<boost::intrusive_ptr<const torrent_info> >();
+    implicitly_convertible<boost::shared_ptr<torrent_info>, boost::shared_ptr<const torrent_info> >();
+    boost::python::register_ptr_to_python<boost::shared_ptr<const torrent_info> >();
 #endif
 }
 
