@@ -2,8 +2,9 @@
 // subject to the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/python.hpp>
-#include <libtorrent/torrent_handle.hpp>
+#include "boost_python.hpp"
+#include <libtorrent/torrent_status.hpp>
+#include <libtorrent/torrent_info.hpp>
 #include <libtorrent/bitfield.hpp>
 
 using namespace boost::python;
@@ -27,6 +28,7 @@ void bind_torrent_status()
         .def_readonly("info_hash", &torrent_status::info_hash)
         .def_readonly("state", &torrent_status::state)
         .def_readonly("paused", &torrent_status::paused)
+        .def_readonly("stop_when_ready", &torrent_status::stop_when_ready)
         .def_readonly("auto_managed", &torrent_status::auto_managed)
         .def_readonly("sequential_download", &torrent_status::sequential_download)
         .def_readonly("is_seeding", &torrent_status::is_seeding)
@@ -40,12 +42,14 @@ void bind_torrent_status()
                 &torrent_status::next_announce, return_value_policy<return_by_value>()
             )
         )
+#ifndef TORRENT_NO_DEPRECATE
         .add_property(
             "announce_interval"
           , make_getter(
                 &torrent_status::announce_interval, return_value_policy<return_by_value>()
             )
         )
+#endif
         .def_readonly("current_tracker", &torrent_status::current_tracker)
         .def_readonly("total_download", &torrent_status::total_download)
         .def_readonly("total_upload", &torrent_status::total_upload)
@@ -89,12 +93,17 @@ void bind_torrent_status()
         .def_readonly("seed_rank", &torrent_status::seed_rank)
         .def_readonly("last_scrape", &torrent_status::last_scrape)
         .def_readonly("has_incoming", &torrent_status::has_incoming)
-        .def_readonly("sparse_regions", &torrent_status::sparse_regions)
         .def_readonly("seed_mode", &torrent_status::seed_mode)
         .def_readonly("upload_mode", &torrent_status::upload_mode)
         .def_readonly("share_mode", &torrent_status::share_mode)
         .def_readonly("super_seeding", &torrent_status::super_seeding)
+#ifndef TORRENT_NO_DEPRECATE
         .def_readonly("error", &torrent_status::error)
+#endif
+        .def_readonly("errc", &torrent_status::errc)
+        .def_readonly("error_file", &torrent_status::error_file)
+        .def_readonly("name", &torrent_status::name)
+        .def_readonly("save_path", &torrent_status::save_path)
         .def_readonly("priority", &torrent_status::priority)
         .def_readonly("added_time", &torrent_status::added_time)
         .def_readonly("completed_time", &torrent_status::completed_time)
@@ -104,10 +113,18 @@ void bind_torrent_status()
         .def_readonly("queue_position", &torrent_status::queue_position)
         .def_readonly("need_save_resume", &torrent_status::need_save_resume)
         .def_readonly("ip_filter_applies", &torrent_status::ip_filter_applies)
+        .def_readonly("moving_storage", &torrent_status::moving_storage)
+        .def_readonly("is_loaded", &torrent_status::is_loaded)
+        .def_readonly("announcing_to_trackers", &torrent_status::announcing_to_trackers)
+        .def_readonly("announcing_to_lsd", &torrent_status::announcing_to_lsd)
+        .def_readonly("announcing_to_dht", &torrent_status::announcing_to_dht)
+        .def_readonly("info_hash", &torrent_status::info_hash)
         ;
 
     enum_<torrent_status::state_t>("states")
+#ifndef TORRENT_NO_DEPRECATE
         .value("queued_for_checking", torrent_status::queued_for_checking)
+#endif
         .value("checking_files", torrent_status::checking_files)
         .value("downloading_metadata", torrent_status::downloading_metadata)
         .value("downloading", torrent_status::downloading)
