@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2005-2014, Arvid Norberg
+Copyright (c) 2005-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef TORRENT_CONFIG_HPP_INCLUDED
 #define TORRENT_CONFIG_HPP_INCLUDED
 
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
+#define _FILE_OFFSET_BITS 64
+
 #if !defined _MSC_VER || _MSC_VER >= 1600
 #ifndef __STDC_LIMIT_MACROS
 #define __STDC_LIMIT_MACROS 1
@@ -42,7 +46,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #endif
 #endif
 
+
 #include <boost/config.hpp>
+#include <boost/asio/detail/config.hpp>
 #include <boost/version.hpp>
 #include <boost/detail/endian.hpp>
 #include <stdio.h> // for snprintf
@@ -50,6 +56,10 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/export.hpp"
 
+#ifdef __linux__
+#include <linux/version.h> // for LINUX_VERSION_CODE and KERNEL_VERSION
+#endif // __linux
+
 #if defined TORRENT_DEBUG_BUFFERS && !defined TORRENT_DISABLE_POOL_ALLOCATOR
 #error TORRENT_DEBUG_BUFFERS only works if you also disable pool allocators with TORRENT_DISABLE_POOL_ALLOCATOR
 #endif
@@ -80,11 +90,16 @@ POSSIBILITY OF SUCH DAMAGE.
 #endif
 #endif
 
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
 // ======= GCC =========
 
 #if defined __GNUC__
 
-# if __GNUC__ >= 3
+// deprecation markup is only enabled when libtorrent
+// headers are included by clients, not while building
+// libtorrent itself
+# if __GNUC__ >= 3 && !defined TORRENT_BUILDING_LIBRARY
 #  define TORRENT_DEPRECATED __attribute__ ((deprecated))
 # endif
 
@@ -109,7 +124,12 @@ POSSIBILITY OF SUCH DAMAGE.
 // '_vsnprintf': This function or variable may be unsafe
 #pragma warning(disable:4996)
 
-#define TORRENT_DEPRECATED_PREFIX __declspec(deprecated)
+// deprecation markup is only enabled when libtorrent
+// headers are included by clients, not while building
+// libtorrent itself
+#if !defined TORRENT_BUILDING_LIBRARY
+# define TORRENT_DEPRECATED __declspec(deprecated)
+#endif
 
 #endif
 
@@ -121,9 +141,6 @@ POSSIBILITY OF SUCH DAMAGE.
 // ==== AMIGA ===
 #if defined __AMIGA__ || defined __amigaos__ || defined __AROS__
 #define TORRENT_AMIGA
-#define TORRENT_USE_MLOCK 0
-#define TORRENT_USE_WRITEV 0
-#define TORRENT_USE_READV 0
 #define TORRENT_USE_IPV6 0
 #define TORRENT_USE_BOOST_THREAD 0
 #define TORRENT_USE_IOSTREAM 0
@@ -143,17 +160,19 @@ POSSIBILITY OF SUCH DAMAGE.
 // we don't need iconv on mac, because
 // the locale is always utf-8
 #if defined __APPLE__
-#ifndef TORRENT_USE_ICONV
-#define TORRENT_USE_ICONV 0
-#define TORRENT_USE_LOCALE 0
-#define TORRENT_CLOSE_MAY_BLOCK 1
 
+# define TORRENT_USE_OSATOMIC 1
+# ifndef TORRENT_USE_ICONV
+#  define TORRENT_USE_ICONV 0
+#  define TORRENT_USE_LOCALE 0
+# endif
 #include <AvailabilityMacros.h>
 
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
-#ifdef TORRENT_USE_OPENSSL
-#define TORRENT_USE_COMMONCRYPTO 1
-#endif // TORRENT_USE_OPENSSL
+// on OSX, use the built-in common crypto for built-in
+# if !defined TORRENT_USE_OPENSSL && !defined TORRENT_USE_GCRYPT
+#  define TORRENT_USE_COMMONCRYPTO 1
+# endif // TORRENT_USE_OPENSSL
 #endif // MAC_OS_X_VERSION_MIN_REQUIRED
 
 // execinfo.h is available in the MacOS X 10.5 SDK.
@@ -161,16 +180,18 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_USE_EXECINFO 1
 #endif
 
-#endif // __APPLE__
-
-#else
+#else // __APPLE__
 // FreeBSD has a reasonable iconv signature
 // unless we're on glibc
 #ifndef __GLIBC__
 # define TORRENT_ICONV_ARG (const char**)
 #endif
-#endif
+#endif // __APPLE__
+
+#define TORRENT_HAVE_MMAP 1
+
 #define TORRENT_HAS_FALLOCATE 0
+
 #define TORRENT_USE_IFADDRS 1
 #define TORRENT_USE_SYSCTL 1
 #define TORRENT_USE_IFCONF 1
@@ -179,6 +200,16 @@ POSSIBILITY OF SUCH DAMAGE.
 // ==== LINUX ===
 #elif defined __linux__
 #define TORRENT_LINUX
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,30)
+# define TORRENT_USE_PREADV 1
+# define TORRENT_USE_PREAD 0
+#else
+# define TORRENT_USE_PREADV 0
+# define TORRENT_USE_PREAD 1
+#endif
+
+#define TORRENT_HAVE_MMAP 1
 #define TORRENT_USE_NETLINK 1
 #define TORRENT_USE_IFCONF 1
 #define TORRENT_HAS_SALEN 0
@@ -190,12 +221,23 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_USE_ICONV 0
 #define TORRENT_USE_IFADDRS 0
 #define TORRENT_USE_MEMALIGN 1
-#else
+#define TORRENT_USE_FDATASYNC 0
+#else // ANDROID
 #define TORRENT_USE_IFADDRS 1
 #define TORRENT_USE_POSIX_MEMALIGN 1
+#define TORRENT_USE_FDATASYNC 1
+
+// posix_fallocate() is available under this condition
+#if _XOPEN_SOURCE >= 600 || _POSIX_C_SOURCE >= 200112L
+#define TORRENT_HAS_FALLOCATE 1
+#else
+#define TORRENT_HAS_FALLOCATE 0
 #endif
 
-#if __amd64__ || __i386__
+#endif // ANDROID
+
+#if defined __GLIBC__ && ( defined __x86_64__ || defined __i386 \
+	|| defined _M_X64 || defined _M_IX86 )
 #define TORRENT_USE_EXECINFO 1
 #endif
 
@@ -212,42 +254,62 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_USE_GETADAPTERSADDRESSES 1
 #define TORRENT_HAS_SALEN 0
 #define TORRENT_USE_GETIPFORWARDTABLE 1
+#define TORRENT_USE_INTERLOCKED_ATOMIC 1
 #ifndef TORRENT_USE_UNC_PATHS
 # define TORRENT_USE_UNC_PATHS 1
 #endif
+// these are emulated on windows
+#define TORRENT_USE_PREADV 1
+#define TORRENT_USE_PWRITEV 1
 
 // ==== WINDOWS ===
 #elif defined WIN32
 #define TORRENT_WINDOWS
 #ifndef TORRENT_USE_GETIPFORWARDTABLE
-#define TORRENT_USE_GETIPFORWARDTABLE 1
+# define TORRENT_USE_GETIPFORWARDTABLE 1
 #endif
 #define TORRENT_USE_GETADAPTERSADDRESSES 1
 #define TORRENT_HAS_SALEN 0
 // windows has its own functions to convert
 #ifndef TORRENT_USE_ICONV
-#define TORRENT_USE_ICONV 0
-#define TORRENT_USE_LOCALE 1
+# define TORRENT_USE_ICONV 0
+# define TORRENT_USE_LOCALE 1
 #endif
 #define TORRENT_USE_RLIMIT 0
 #define TORRENT_HAS_FALLOCATE 0
+#define TORRENT_USE_INTERLOCKED_ATOMIC 1
 #ifndef TORRENT_USE_UNC_PATHS
-#define TORRENT_USE_UNC_PATHS 1
+# define TORRENT_USE_UNC_PATHS 1
+#endif
+// these are emulated on windows
+#define TORRENT_USE_PREADV 1
+#define TORRENT_USE_PWRITEV 1
+
+// ==== WINRT ===
+#if defined(WINAPI_FAMILY_PARTITION)
+# if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP) \
+  && !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#  define TORRENT_WINRT
+#  define TORRENT_USE_CRYPTOGRAPHIC_BUFFER 1
+# endif
 #endif
 
 // ==== SOLARIS ===
-#elif defined sun || defined __sun 
+#elif defined sun || defined __sun
 #define TORRENT_SOLARIS
 #define TORRENT_COMPLETE_TYPES_REQUIRED 1
 #define TORRENT_USE_IFCONF 1
 #define TORRENT_HAS_SALEN 0
+#define TORRENT_HAS_SEM_RELTIMEDWAIT 1
+#define TORRENT_HAVE_MMAP 1
+#define TORRENT_USE_SOLARIS_ATOMIC 1
 
 // ==== BEOS ===
 #elif defined __BEOS__ || defined __HAIKU__
 #define TORRENT_BEOS
 #include <storage/StorageDefs.h> // B_PATH_NAME_LENGTH
 #define TORRENT_HAS_FALLOCATE 0
-#define TORRENT_USE_MLOCK 0
+#define TORRENT_USE_BEOS_ATOMIC 1
 #ifndef TORRENT_USE_ICONV
 #define TORRENT_USE_ICONV 0
 #endif
@@ -264,7 +326,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_HAS_FALLOCATE 0
 #define TORRENT_USE_IFCONF 1
 #define TORRENT_USE_SYSCTL 1
-#define TORRENT_USE_MLOCK 0
 #define TORRENT_USE_IPV6 0
 #define TORRENT_ICONV_ARG (const char**)
 #define TORRENT_USE_WRITEV 0
@@ -281,6 +342,16 @@ POSSIBILITY OF SUCH DAMAGE.
 #define TORRENT_BSD
 #endif
 
+#if defined __GNUC__ && !(defined TORRENT_USE_OSATOMIC \
+	|| defined TORRENT_USE_INTERLOCKED_ATOMIC \
+	|| defined TORRENT_USE_BEOS_ATOMIC \
+	|| defined TORRENT_USE_SOLARIS_ATOMIC)
+// atomic operations in GCC were introduced in 4.1.1
+# if (__GNUC__ >= 4 && __GNUC_MINOR__ >= 1 && __GNUC_PATCHLEVEL__ >= 1) || __GNUC__ > 4
+#  define TORRENT_USE_GCC_ATOMIC 1
+# endif
+#endif
+
 // on windows, NAME_MAX refers to Unicode characters
 // on linux it refers to bytes (utf-8 encoded)
 // TODO: Make this count Unicode characters instead of bytes on windows
@@ -297,14 +368,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #elif defined MAXPATH
 #define TORRENT_MAX_PATH MAXPATH
 
-// posix
-#elif defined NAME_MAX
-#define TORRENT_MAX_PATH NAME_MAX
-
 // none of the above
 #else
 // this is the maximum number of characters in a
-// path element / filename on windows
+// path element / filename on windows and also on many filesystems commonly used
+// on linux
 #define TORRENT_MAX_PATH 255
 
 #ifdef _MSC_VER
@@ -315,7 +383,9 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #endif
 
-#if defined TORRENT_WINDOWS && !defined TORRENT_MINGW
+#define TORRENT_UNUSED(x) (void)(x)
+
+#if (defined _MSC_VER && _MSC_VER < 1900) && !defined TORRENT_MINGW
 
 #include <stdarg.h>
 
@@ -337,20 +407,65 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 }
 
 #define strtoll _strtoi64
-#else
-#include <limits.h>
 #endif
 
-#if (defined(TORRENT_LOGGING) || defined(TORRENT_VERBOSE_LOGGING)) \
-	&& !defined (TORRENT_UPNP_LOGGING) && TORRENT_USE_IOSTREAM
-#define TORRENT_UPNP_LOGGING
+// at the highest warning level, clang actually warns about functions
+// that could be marked noreturn.
+#if defined __clang__ || defined __GNUC__
+#define TORRENT_NO_RETURN __attribute((noreturn))
+#else
+#define TORRENT_NO_RETURN
+#endif
+
+#ifdef _GLIBCXX_USE_NOEXCEPT
+#define TORRENT_EXCEPTION_THROW_SPECIFIER _GLIBCXX_USE_NOEXCEPT
+#else
+#if __cplusplus <= 199711L || defined BOOST_NO_CXX11_NOEXCEPT
+#define TORRENT_EXCEPTION_THROW_SPECIFIER throw()
+#else
+#define TORRENT_EXCEPTION_THROW_SPECIFIER noexcept
+#endif
+#endif // __GLIBC__
+
+#if __cplusplus <= 199711L || defined BOOST_NO_CXX11_FINAL
+#define TORRENT_FINAL
+#else
+#define TORRENT_FINAL final
+#endif
+
+#if __cplusplus <= 199711L || defined BOOST_NO_CXX11_FINAL
+#define TORRENT_OVERRIDE
+#else
+#define TORRENT_OVERRIDE override
 #endif
 
 #ifndef TORRENT_ICONV_ARG
 #define TORRENT_ICONV_ARG (char**)
 #endif
 
-// libiconv presence, not implemented yet
+#if defined __GNUC__ || defined __clang__
+#define TORRENT_FORMAT(fmt, ellipsis) __attribute__((__format__(__printf__, fmt, ellipsis)))
+#else
+#define TORRENT_FORMAT(fmt, ellipsis)
+#endif
+
+#ifndef TORRENT_USE_INTERLOCKED_ATOMIC
+#define TORRENT_USE_INTERLOCKED_ATOMIC 0
+#endif
+
+#ifndef TORRENT_USE_GCC_ATOMIC
+#define TORRENT_USE_GCC_ATOMIC 0
+#endif
+
+#ifndef TORRENT_USE_OSATOMIC
+#define TORRENT_USE_OSATOMIC 0
+#endif
+
+#ifndef TORRENT_USE_BEOS_ATOMIC
+#define TORRENT_USE_BEOS_ATOMIC 0
+#endif
+
+// libiconv presence detection is not implemented yet
 #ifndef TORRENT_USE_ICONV
 #define TORRENT_USE_ICONV 1
 #endif
@@ -379,16 +494,20 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #define TORRENT_USE_GETIPFORWARDTABLE 0
 #endif
 
-#ifndef TORRENT_USE_LOCALE
-#define TORRENT_USE_LOCALE 0
+#ifndef TORRENT_HAS_SEM_RELTIMEDWAIT
+#define TORRENT_HAS_SEM_RELTIMEDWAIT 0
 #endif
 
-// set this to true if close() may block on your system
-// Mac OS X does this if the file being closed is not fully
-// allocated on disk yet for instance. When defined, the disk
-// I/O subsytem will use a separate thread for closing files
-#ifndef TORRENT_CLOSE_MAY_BLOCK
-#define TORRENT_CLOSE_MAY_BLOCK 0
+#ifndef TORRENT_USE_MEMALIGN
+#define TORRENT_USE_MEMALIGN 0
+#endif
+
+#ifndef TORRENT_USE_POSIX_MEMALIGN
+#define TORRENT_USE_POSIX_MEMALIGN 0
+#endif
+
+#ifndef TORRENT_USE_LOCALE
+#define TORRENT_USE_LOCALE 0
 #endif
 
 #ifndef TORRENT_BROKEN_UNIONS
@@ -407,22 +526,26 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #define TORRENT_HAS_FALLOCATE 1
 #endif
 
-#ifndef TORRENT_DEPRECATED_PREFIX
-#define TORRENT_DEPRECATED_PREFIX
+#ifndef TORRENT_DEPRECATED
+#define TORRENT_DEPRECATED
 #endif
 
 #ifndef TORRENT_USE_COMMONCRYPTO
 #define TORRENT_USE_COMMONCRYPTO 0
 #endif
 
-#ifndef TORRENT_DEPRECATED
-#define TORRENT_DEPRECATED
+#ifndef TORRENT_HAVE_MMAP
+#define TORRENT_HAVE_MMAP 0
 #endif
 
 #ifndef TORRENT_COMPLETE_TYPES_REQUIRED
 #define TORRENT_COMPLETE_TYPES_REQUIRED 0
 #endif
 
+#ifndef TORRENT_USE_FDATASYNC
+#define TORRENT_USE_FDATASYNC 0
+#endif
+
 #ifndef TORRENT_USE_UNC_PATHS
 #define TORRENT_USE_UNC_PATHS 0
 #endif
@@ -439,16 +562,14 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #define TORRENT_USE_IPV6 1
 #endif
 
-#ifndef TORRENT_USE_MLOCK
-#define TORRENT_USE_MLOCK 1
+// if preadv() exists, we assume pwritev() does as well
+#ifndef TORRENT_USE_PREADV
+#define TORRENT_USE_PREADV 0
 #endif
 
-#ifndef TORRENT_USE_WRITEV
-#define TORRENT_USE_WRITEV 1
-#endif
-
-#ifndef TORRENT_USE_READV
-#define TORRENT_USE_READV 1
+// if pread() exists, we assume pwrite() does as well
+#ifndef TORRENT_USE_PREAD
+#define TORRENT_USE_PREAD 1
 #endif
 
 #ifndef TORRENT_NO_FPU
@@ -463,16 +584,24 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #endif
 #endif
 
-// if set to true, piece picker will use less RAM
-// but only support up to ~260000 pieces in a torrent
-#ifndef TORRENT_COMPACT_PICKER
-#define TORRENT_COMPACT_PICKER 0
+// whether function-local static variables are thread safe. In c++11 and later
+// they are (except msvc)
+#ifndef TORRENT_THREADSAFE_STATIC
+#if __cplusplus < 199711L || (defined _MSC_VER && _MSC_VER <= 1800)
+#define TORRENT_THREADSAFE_STATIC 0
+#else
+#define TORRENT_THREADSAFE_STATIC 1
+#endif
 #endif
 
 #ifndef TORRENT_USE_I2P
 #define TORRENT_USE_I2P 1
 #endif
 
+#ifndef TORRENT_HAS_BOOST_UNORDERED
+#define TORRENT_HAS_BOOST_UNORDERED 1
+#endif
+
 #if !defined TORRENT_IOV_MAX
 #ifdef IOV_MAX
 #define TORRENT_IOV_MAX IOV_MAX
@@ -500,7 +629,16 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #endif
 
 #if defined _MSC_VER && _MSC_VER <= 1200
-#define for if (false) {} else for
+// this is here to provide a standard-conforming for
+// keyword for old versions of msvc. The pragmas are
+// there to silence the warning it produces by using
+// a constant as conditional
+#define for \
+	__pragma( warning(push) ) \
+	__pragma( warning(disable:4127) ) \
+	if (false) {} else \
+	__pragma( warning(pop) )
+	for
 #endif
 
 #if TORRENT_BROKEN_UNIONS
@@ -509,38 +647,18 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #define TORRENT_UNION union
 #endif
 
-// determine what timer implementation we can use
-// if one is already defined, don't pick one
-// autmatically. This lets the user control this
-// from the Jamfile
-#if !defined TORRENT_USE_ABSOLUTE_TIME \
-	&& !defined TORRENT_USE_QUERY_PERFORMANCE_TIMER \
-	&& !defined TORRENT_USE_CLOCK_GETTIME \
-	&& !defined TORRENT_USE_BOOST_DATE_TIME \
-	&& !defined TORRENT_USE_ECLOCK \
-	&& !defined TORRENT_USE_SYSTEM_TIME
-
-#if defined __APPLE__ && defined __MACH__
-#define TORRENT_USE_ABSOLUTE_TIME 1
-#elif defined(_WIN32) || defined TORRENT_MINGW
-#define TORRENT_USE_QUERY_PERFORMANCE_TIMER 1
-#elif defined(_POSIX_MONOTONIC_CLOCK) && _POSIX_MONOTONIC_CLOCK >= 0
-#define TORRENT_USE_CLOCK_GETTIME 1
-#elif defined(TORRENT_AMIGA)
-#define TORRENT_USE_ECLOCK 1
-#elif defined(TORRENT_BEOS)
-#define TORRENT_USE_SYSTEM_TIME 1
+#if defined __GNUC__
+#define TORRENT_FUNCTION __PRETTY_FUNCTION__
 #else
-#define TORRENT_USE_BOOST_DATE_TIME 1
+#define TORRENT_FUNCTION __FUNCTION__
 #endif
 
-#endif
 
 // debug builds have asserts enabled by default, release
 // builds have asserts if they are explicitly enabled by
 // the release_asserts macro.
 #ifndef TORRENT_USE_ASSERTS
-#if defined TORRENT_DEBUG || TORRENT_RELEASE_ASSERTS
+#if defined TORRENT_DEBUG || defined TORRENT_RELEASE_ASSERTS
 #define TORRENT_USE_ASSERTS 1
 #else
 #define TORRENT_USE_ASSERTS 0
@@ -567,6 +685,24 @@ int snprintf(char* buf, int len, char const* fmt, ...)
 #define TORRENT_DECLARE_DUMMY(x, y)
 #endif // BOOST_NO_EXCEPTIONS
 
+// SSE is x86 / amd64 specific. On top of that, we only
+// know how to access it on msvc and gcc (and gcc compatibles).
+// GCC requires the user to enable SSE support in order for
+// the program to have access to the intrinsics, this is
+// indicated by the __SSE4_1__ macro
+#ifndef TORRENT_HAS_SSE
+
+#if (defined _M_AMD64 || defined _M_IX86 || defined _M_X64 \
+	|| defined __amd64__ || defined __i386 || defined __i386__ \
+	|| defined __x86_64__ || defined __x86_64) \
+	&& (defined __GNUC__ || defined _MSC_VER)
+#define TORRENT_HAS_SSE 1
+#else
+#define TORRENT_HAS_SSE 0
+#endif
+
+#endif // TORRENT_HAS_SSE
+
 
 #endif // TORRENT_CONFIG_HPP_INCLUDED
 
