@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2007, Arvid Norberg
+Copyright (c) 2007-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -43,33 +43,40 @@ namespace libtorrent
 	struct torrent_handle;
 	class session;
 
+	// Generates a magnet URI from the specified torrent. If the torrent
+	// handle is invalid, an empty string is returned.
+	// 
+	// For more information about magnet links, see magnet-links_.
+	// 
 	std::string TORRENT_EXPORT make_magnet_uri(torrent_handle const& handle);
 	std::string TORRENT_EXPORT make_magnet_uri(torrent_info const& info);
 
 #ifndef TORRENT_NO_DEPRECATE
 #ifndef BOOST_NO_EXCEPTIONS
 	// deprecated in 0.14
-	TORRENT_DEPRECATED_PREFIX
+	TORRENT_DEPRECATED
 	torrent_handle TORRENT_EXPORT add_magnet_uri(session& ses, std::string const& uri
 		, std::string const& save_path
 		, storage_mode_t storage_mode = storage_mode_sparse
 		, bool paused = false
 		, storage_constructor_type sc = default_storage_constructor
-		, void* userdata = 0) TORRENT_DEPRECATED;
+		, void* userdata = 0);
 
 	// deprecated in 0.16. Instead, pass in the magnet link as add_torrent_params::url
-	TORRENT_DEPRECATED_PREFIX
+	TORRENT_DEPRECATED
 	torrent_handle TORRENT_EXPORT add_magnet_uri(session& ses, std::string const& uri
-		, add_torrent_params p) TORRENT_DEPRECATED;
+		, add_torrent_params const& p);
 #endif
 
 	// deprecated in 0.16. Instead, pass in the magnet link as add_torrent_params::url
-	TORRENT_DEPRECATED_PREFIX
+	TORRENT_DEPRECATED
 	torrent_handle TORRENT_EXPORT add_magnet_uri(session& ses, std::string const& uri
-		, add_torrent_params p, error_code& ec) TORRENT_DEPRECATED;
+		, add_torrent_params const& p, error_code& ec);
 
 #endif
 
+	// This function parses out information from the magnet link and populates the
+	// add_torrent_params object.
 	TORRENT_EXPORT void parse_magnet_uri(std::string const& uri, add_torrent_params& p, error_code& ec);
 }
 
