@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2014, Arvid Norberg
+Copyright (c) 2003-2016, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -30,20 +30,25 @@ POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-#include <algorithm>
+#include "libtorrent/config.hpp"
+
+#include "libtorrent/aux_/disable_warnings_push.hpp"
+
 #if TORRENT_USE_IOSTREAM
 #include <iostream>
 #endif
+#include <algorithm>
 #include <boost/bind.hpp>
-#include "libtorrent/entry.hpp"
-#include "libtorrent/config.hpp"
-#include "libtorrent/escape_string.hpp"
-#include "libtorrent/lazy_entry.hpp"
-#include "libtorrent/escape_string.hpp"
 
-#if defined(_MSC_VER)
-#define for if (false) {} else for
+#include "libtorrent/aux_/disable_warnings_pop.hpp"
+
+#include "libtorrent/entry.hpp"
+#ifndef TORRENT_NO_DEPRECATE
+#include "libtorrent/lazy_entry.hpp"
 #endif
+#include "libtorrent/bdecode.hpp"
+#include "libtorrent/entry.hpp"
+#include "libtorrent/hex.hpp"
 
 namespace
 {
@@ -59,7 +64,8 @@ namespace libtorrent
 {
 	namespace detail
 	{
-		TORRENT_EXPORT char const* integer_to_str(char* buf, int size, entry::integer_type val)
+		TORRENT_EXPORT char const* integer_to_str(char* buf, int size
+			, entry::integer_type val)
 		{
 			int sign = 0;
 			if (val < 0)
@@ -128,15 +134,15 @@ namespace libtorrent
 #ifndef BOOST_NO_EXCEPTIONS
 	const entry& entry::operator[](char const* key) const
 	{
-		dictionary_type::const_iterator i = dict().find(key);
-		if (i == dict().end()) throw type_error(
-			(std::string("key not found: ") + key).c_str());
-		return i->second;
+		return (*this)[std::string(key)];
 	}
 
 	const entry& entry::operator[](std::string const& key) const
 	{
-		return (*this)[key.c_str()];
+		dictionary_type::const_iterator i = dict().find(key);
+		if (i == dict().end()) throw type_error(
+			(std::string("key not found: ") + key).c_str());
+		return i->second;
 	}
 #endif
 
@@ -145,7 +151,7 @@ namespace libtorrent
 #ifdef TORRENT_DEBUG
 		m_type_queried = true;
 #endif
-		return (entry::data_type)m_type;
+		return entry::data_type(m_type);
 	}
 
 	entry::~entry() { destruct(); }
@@ -314,6 +320,44 @@ namespace libtorrent
 		m_type = int_t;
 	}
 
+	// convert a bdecode_node into an old skool entry
+	void entry::operator=(bdecode_node const& e)
+	{
+		switch (e.type())
+		{
+			case bdecode_node::string_t:
+				this->string() = e.string_value();
+				break;
+			case bdecode_node::int_t:
+				this->integer() = e.int_value();
+				break;
+			case bdecode_node::dict_t:
+			{
+				dictionary_type& d = this->dict();
+				for (int i = 0; i < e.dict_size(); ++i)
+				{
+					std::pair<std::string, bdecode_node> elem = e.dict_at(i);
+					d[elem.first] = elem.second;
+				}
+				break;
+			}
+			case bdecode_node::list_t:
+			{
+				list_type& l = this->list();
+				for (int i = 0; i < e.list_size(); ++i)
+				{
+					l.push_back(entry());
+					l.back() = e.list_at(i);
+				}
+				break;
+			}
+			case bdecode_node::none_t:
+				destruct();
+				break;
+		}
+	}
+
+#ifndef TORRENT_NO_DEPRECATE
 	// convert a lazy_entry into an old skool entry
 	void entry::operator=(lazy_entry const& e)
 	{
@@ -350,6 +394,7 @@ namespace libtorrent
 				break;
 		}
 	}
+#endif
 
 	void entry::operator=(dictionary_type const& v)
 	{
@@ -395,7 +440,7 @@ namespace libtorrent
 	{
 		if (m_type != e.m_type) return false;
 
-		switch(m_type)
+		switch (m_type)
 		{
 		case int_t:
 			return integer() == e.integer();
@@ -413,7 +458,7 @@ namespace libtorrent
 
 	void entry::construct(data_type t)
 	{
-		switch(t)
+		switch (t)
 		{
 		case int_t:
 			new(data) integer_type;
@@ -427,8 +472,8 @@ namespace libtorrent
 		case dictionary_t:
 			new (data) dictionary_type;
 			break;
-		default:
-			TORRENT_ASSERT(t == undefined_t);
+		case undefined_t:
+			break;
 		}
 		m_type = t;
 #ifdef TORRENT_DEBUG
@@ -452,7 +497,7 @@ namespace libtorrent
 		case dictionary_t:
 			new (data) dictionary_type(e.dict());
 			break;
-		default:
+		case undefined_t:
 			TORRENT_ASSERT(e.type() == undefined_t);
 		}
 		m_type = e.type();
@@ -497,13 +542,13 @@ namespace libtorrent
 
 		if (m_type == undefined_t)
 		{
-			construct((data_type)e.m_type);
+			construct(data_type(e.m_type));
 			clear_that = true;
 		}
 
 		if (e.m_type == undefined_t)
 		{
-			e.construct((data_type)m_type);
+			e.construct(data_type(m_type));
 			clear_this = true;
 		}
 
@@ -541,7 +586,7 @@ namespace libtorrent
 		{
 			// currently, only swapping entries of the same type or where one
 			// of the entries is uninitialized is supported.
-			TORRENT_ASSERT(false && "not implemented");
+			TORRENT_ASSERT(false);
 		}
 	}
 
@@ -560,7 +605,7 @@ namespace libtorrent
 		{
 		case int_t:
 			out += libtorrent::to_string(integer()).elems;
-		  	out += "\n";
+			out += "\n";
 			break;
 		case string_t:
 			{
@@ -619,6 +664,7 @@ namespace libtorrent
 					i->second.to_string_impl(out, indent+2);
 				}
 			} break;
+		case undefined_t:
 		default:
 			out += "<uninitialized>\n";
 		}
