@@ -37,7 +37,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/io_service.hpp"
 #include "libtorrent/error_code.hpp"
 #include "libtorrent/socket.hpp"
+#include "libtorrent/aux_/time.hpp"
+#include "libtorrent/io_service.hpp"
 #include "peer_server.hpp"
+#include "test_utils.hpp"
 
 #include <boost/detail/atomic_count.hpp>
 #include <boost/shared_ptr.hpp>
@@ -48,7 +51,7 @@ using namespace libtorrent;
 struct peer_server
 {
 
-	boost::asio::io_service m_ios;
+	libtorrent::io_service m_ios;
 	boost::detail::atomic_count m_peer_requests;
 	tcp::acceptor m_acceptor;
 	int m_port;
@@ -64,30 +67,30 @@ struct peer_server
 		m_acceptor.open(tcp::v4(), ec);
 		if (ec)
 		{
-			fprintf(stderr, "Error opening peer listen socket: %s\n", ec.message().c_str());
+			fprintf(stderr, "PEER Error opening peer listen socket: %s\n", ec.message().c_str());
 			return;
 		}
 
 		m_acceptor.bind(tcp::endpoint(address_v4::any(), 0), ec);
 		if (ec)
 		{
-			fprintf(stderr, "Error binding peer socket to port 0: %s\n", ec.message().c_str());
+			fprintf(stderr, "PEER Error binding peer socket to port 0: %s\n", ec.message().c_str());
 			return;
 		}
 		m_port = m_acceptor.local_endpoint(ec).port();
 		if (ec)
 		{
-			fprintf(stderr, "Error getting local endpoint of peer socket: %s\n", ec.message().c_str());
+			fprintf(stderr, "PEER Error getting local endpoint of peer socket: %s\n", ec.message().c_str());
 			return;
 		}
 		m_acceptor.listen(10, ec);
 		if (ec)
 		{
-			fprintf(stderr, "Error listening on peer socket: %s\n", ec.message().c_str());
+			fprintf(stderr, "PEER Error listening on peer socket: %s\n", ec.message().c_str());
 			return;
 		}
 
-		fprintf(stderr, "%s: peer initialized on port %d\n", time_now_string(), m_port);
+		fprintf(stderr, "%s: PEER peer initialized on port %d\n", time_now_string(), m_port);
 
 		m_thread.reset(new thread(boost::bind(&peer_server::thread_fun, this)));
 	}
@@ -121,7 +124,7 @@ struct peer_server
 			m_acceptor.async_accept(socket, from, boost::bind(&new_connection, _1, &ec, &done));
 			while (!done)
 			{
-				m_ios.run_one();
+				m_ios.poll_one();
 				m_ios.reset();
 			}
 
@@ -130,11 +133,11 @@ struct peer_server
 
 			if (ec)
 			{
-				fprintf(stderr, "Error accepting connection on peer socket: %s\n", ec.message().c_str());
+				fprintf(stderr, "PEER Error accepting connection on peer socket: %s\n", ec.message().c_str());
 				return;
 			}
 
-			fprintf(stderr, "%s: incoming peer connection\n", time_now_string());
+			fprintf(stderr, "%s: PEER incoming peer connection\n", time_now_string());
 			++m_peer_requests;
 			socket.close(ec);
 		}
@@ -158,8 +161,6 @@ int num_peer_hits()
 
 void stop_peer()
 {
-	fprintf(stderr, "%s: stop_peer()\n", time_now_string());
 	g_peer.reset();
-	fprintf(stderr, "%s: stop_peer() done\n", time_now_string());
 }
 
