@@ -38,31 +38,26 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/thread.hpp"
 #include "libtorrent/time.hpp"
 #include "libtorrent/file.hpp"
+#include "libtorrent/torrent_info.hpp"
 #include <boost/tuple/tuple.hpp>
 #include <boost/bind.hpp>
 
 #include "test.hpp"
 #include "setup_transfer.hpp"
-#include "udp_tracker.hpp"
 #include <fstream>
 #include <iostream>
 
 using namespace libtorrent;
+namespace lt = libtorrent;
 using boost::tuples::ignore;
 
-int const alert_mask = alert::all_categories
-& ~alert::progress_notification
-& ~alert::stats_notification;
+const int mask = alert::all_categories & ~(alert::performance_warning | alert::stats_notification);
 
 int peer_disconnects = 0;
 
-int tracker_responses = 0;
-
-bool on_alert(alert* a)
+bool on_alert(alert const* a)
 {
-	if (alert_cast<tracker_reply_alert>(a))
-		++tracker_responses;
-	else if (alert_cast<peer_disconnected_alert>(a))
+	if (alert_cast<peer_disconnected_alert>(a))
 		++peer_disconnects;
 	else if (alert_cast<peer_error_alert>(a))
 		++peer_disconnects;
@@ -70,66 +65,63 @@ bool on_alert(alert* a)
 	return false;
 }
 
-int udp_tracker_port;
-int tracker_port;
-
-// these are declared before the session objects
-// so that they are destructed last. This enables
-// the sessions to destruct in parallel
-std::vector<session_proxy> sp;
-
-void test_transfer(session_settings& sett)
+void cleanup()
 {
-	// in case the previous run was terminated
 	error_code ec;
-	remove_all("tmp1_priority", ec);
-	remove_all("tmp2_priority", ec);
-	remove_all("tmp1_priority_moved", ec);
-	remove_all("tmp2_priority_moved", ec);
+	remove_all("tmp1_priorities", ec);
+	remove_all("tmp2_priorities", ec);
+	remove_all("tmp1_priorities_moved", ec);
+	remove_all("tmp2_priorities_moved", ec);
+}
 
-	session ses1(fingerprint("LT", 0, 1, 0, 0), std::make_pair(48075, 49000), "0.0.0.0", 0, alert_mask);
-	session ses2(fingerprint("LT", 0, 1, 0, 0), std::make_pair(49075, 50000), "0.0.0.0", 0, alert_mask);
+void test_transfer(settings_pack const& sett)
+{
+	// this allows shutting down the sessions in parallel
+	std::vector<session_proxy> sp;
+
+	cleanup();
+
+	settings_pack pack = sett;
 
 	// we need a short reconnect time since we
 	// finish the torrent and then restart it
 	// immediately to complete the second half.
 	// using a reconnect time > 0 will just add
 	// to the time it will take to complete the test
-	sett.min_reconnect_time = 0;
-	sett.stop_tracker_timeout = 1;
-	sett.announce_to_all_trackers = true;
-	sett.announce_to_all_tiers = true;
-	// make sure we announce to both http and udp trackers
-	sett.prefer_udp_trackers = false;
-	sett.enable_outgoing_utp = false;
-	sett.enable_incoming_utp = false;
+	pack.set_int(settings_pack::min_reconnect_time, 0);
 
-	ses1.set_settings(sett);
-	ses2.set_settings(sett);
+	pack.set_bool(settings_pack::enable_outgoing_utp, false);
+	pack.set_bool(settings_pack::enable_incoming_utp, false);
+	pack.set_int(settings_pack::alert_mask, mask);
 
-#ifndef TORRENT_DISABLE_ENCRYPTION
-	pe_settings pes;
-	pes.out_enc_policy = pe_settings::disabled;
-	pes.in_enc_policy = pe_settings::disabled;
-	ses1.set_pe_settings(pes);
-	ses2.set_pe_settings(pes);
-#endif
+	pack.set_int(settings_pack::out_enc_policy, settings_pack::pe_disabled);
+	pack.set_int(settings_pack::in_enc_policy, settings_pack::pe_disabled);
+	pack.set_bool(settings_pack::allow_multiple_connections_per_ip, false);
+	pack.set_int(settings_pack::unchoke_slots_limit, 8);
+
+	pack.set_bool(settings_pack::enable_upnp, false);
+	pack.set_bool(settings_pack::enable_natpmp, false);
+	pack.set_bool(settings_pack::enable_lsd, false);
+	pack.set_bool(settings_pack::enable_dht, false);
+
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:48075");
+	pack.set_int(settings_pack::alert_mask, mask);
+
+	lt::session ses1(pack);
+
+	pack.set_str(settings_pack::listen_interfaces, "0.0.0.0:49075");
+	pack.set_int(settings_pack::alert_mask, mask);
+	lt::session ses2(pack);
 
 	torrent_handle tor1;
 	torrent_handle tor2;
 
+	error_code ec;
 	create_directory("tmp1_priority", ec);
 	std::ofstream file("tmp1_priority/temporary");
-	boost::intrusive_ptr<torrent_info> t = ::create_torrent(&file, 16 * 1024, 13, false);
+	boost::shared_ptr<torrent_info> t = ::create_torrent(&file, "temporary", 16 * 1024, 13, false);
 	file.close();
 
-	char tracker_url[200];
-	snprintf(tracker_url, sizeof(tracker_url), "http://127.0.0.1:%d/announce", tracker_port);
-	t->add_tracker(tracker_url);
-
-	snprintf(tracker_url, sizeof(tracker_url), "udp://127.0.0.1:%d/announce", udp_tracker_port);
-	t->add_tracker(tracker_url);
-
 	add_torrent_params addp;
 	addp.flags &= ~add_torrent_params::flag_paused;
 	addp.flags &= ~add_torrent_params::flag_auto_managed;
@@ -152,8 +144,6 @@ void test_transfer(session_settings& sett)
 	std::copy(priorities.begin(), priorities.end(), std::ostream_iterator<int>(std::cerr, ", "));
 	std::cerr << std::endl;
 
-	tracker_responses = 0;
-
 	for (int i = 0; i < 200; ++i)
 	{
 		print_alerts(ses1, "ses1", true, true, true, &on_alert);
@@ -172,7 +162,7 @@ void test_transfer(session_settings& sett)
 
 		if (st2.state != torrent_status::downloading)
 		{
-			static char const* state_str[] =	
+			static char const* state_str[] =
 				{"checking (q)", "checking", "dl metadata"
 				, "downloading", "finished", "seeding", "allocating", "checking (r)"};
 			std::cerr << "st2 state: " << state_str[st2.state] << std::endl;
@@ -191,9 +181,6 @@ void test_transfer(session_settings& sett)
 		test_sleep(100);
 	}
 
-	// 1 announce per tracker to start
-	TEST_CHECK(tracker_responses >= 2);
-
 	TEST_CHECK(!tor2.status().is_seeding);
 	TEST_CHECK(tor2.status().is_finished);
 
@@ -216,7 +203,6 @@ void test_transfer(session_settings& sett)
 
 	peer_disconnects = 0;
 
-	// wait until force-recheck is complete
 	// when we're done checking, we're likely to be put in downloading state
 	// for a split second before transitioning to finished. This loop waits
 	// for the finished state
@@ -235,8 +221,12 @@ void test_transfer(session_settings& sett)
 		test_sleep(100);
 	}
 
-	TEST_CHECK(st2.state != torrent_status::checking_files);
-	if (st2.state != torrent_status::checking_files) std::cerr << "recheck complete" << std::endl;
+	TEST_EQUAL(st2.state, torrent_status::finished);
+
+	if (st2.state != torrent_status::finished)
+		return;
+
+	std::cerr << "recheck complete" << std::endl;
 
 	priorities2 = tor2.piece_priorities();
 	std::copy(priorities2.begin(), priorities2.end(), std::ostream_iterator<int>(std::cerr, ", "));
@@ -244,45 +234,48 @@ void test_transfer(session_settings& sett)
 	TEST_CHECK(std::equal(priorities.begin(), priorities.end(), priorities2.begin()));
 
 	tor2.pause();
-	alert const* a = ses2.wait_for_alert(seconds(10));
-	bool got_paused_alert = false;
-	while (a)
-	{
-		std::auto_ptr<alert> holder = ses2.pop_alert();
-		std::cerr << "ses2: " << a->message() << std::endl;
-		if (alert_cast<torrent_paused_alert>(a))
-		{
-			got_paused_alert = true;
-			break;	
-		}
-		a = ses2.wait_for_alert(seconds(10));
-	}
-	TEST_CHECK(got_paused_alert);	
-
-	std::vector<announce_entry> tr = tor2.trackers();
-	tr.push_back(announce_entry("http://test.com/announce"));
-	tor2.replace_trackers(tr);
-	tr.clear();
+	wait_for_alert(ses2, torrent_paused_alert::alert_type, "ses2");
 
+	fprintf(stderr, "save resume data\n");
 	tor2.save_resume_data();
 
 	std::vector<char> resume_data;
-	a = ses2.wait_for_alert(seconds(10));
-	while (a)
-	{
-		std::auto_ptr<alert> holder = ses2.pop_alert();
-		std::cerr << "ses2: " << a->message() << std::endl;
-		if (alert_cast<save_resume_data_alert>(a))
-		{
-			bencode(std::back_inserter(resume_data)
-					, *alert_cast<save_resume_data_alert>(a)->resume_data);
-			break;
-		}
-		a = ses2.wait_for_alert(seconds(10));
-	}
-	TEST_CHECK(resume_data.size());	
 
-	std::cerr << "saved resume data" << std::endl;
+	time_point start = clock_type::now();
+	while (true)
+	{
+		ses2.wait_for_alert(seconds(10));
+		std::vector<alert*> alerts;
+		ses2.pop_alerts(&alerts);
+		if (alerts.empty()) break;
+		for (std::vector<alert*>::iterator i = alerts.begin()
+			, end(alerts.end()); i != end; ++i)
+		{
+			alert* a = *i;
+			std::cerr << "ses2: " << a->message() << std::endl;
+			if (alert_cast<save_resume_data_alert>(a))
+			{
+				bencode(std::back_inserter(resume_data)
+					, *alert_cast<save_resume_data_alert>(a)->resume_data);
+				fprintf(stderr, "saved resume data\n");
+				goto done;
+			}
+			else if (alert_cast<save_resume_data_failed_alert>(a))
+			{
+				fprintf(stderr, "save resume failed\n");
+				goto done;
+			}
+			if (total_seconds(clock_type::now() - start) > 10)
+				goto done;
+		}
+	}
+done:
+	TEST_CHECK(resume_data.size());
+
+	if (resume_data.empty())
+		return;
+
+	fprintf(stderr, "%s\n", &resume_data[0]);
 
 	ses2.remove_torrent(tor2);
 
@@ -298,35 +291,32 @@ void test_transfer(session_settings& sett)
 	p.save_path = "tmp2_priority";
 	p.resume_data = resume_data;
 	tor2 = ses2.add_torrent(p, ec);
-	ses2.set_alert_mask(alert::all_categories
-		& ~alert::progress_notification
-		& ~alert::stats_notification);
 	tor2.prioritize_pieces(priorities);
 	std::cout << "resetting priorities" << std::endl;
 	tor2.resume();
 
-	tr = tor2.trackers();
-	TEST_CHECK(std::find_if(tr.begin(), tr.end()
-		, boost::bind(&announce_entry::url, _1) == "http://test.com/announce") != tr.end());
-
-	// wait for the files in ses2 to be checked, i.e. the torrent
-	// to turn into finished state
+	// wait for torrent 2 to settle in back to finished state (it will
+	// start as checking)
+	torrent_status st1;
 	for (int i = 0; i < 5; ++i)
 	{
 		print_alerts(ses1, "ses1", true, true, true, &on_alert);
 		print_alerts(ses2, "ses2", true, true, true, &on_alert);
 
-		torrent_status st1 = tor1.status();
-		torrent_status st2 = tor2.status();
+		st1 = tor1.status();
+		st2 = tor2.status();
 
-		TEST_EQUAL(st1.state, torrent_status::seeding);
+		TEST_CHECK(st1.state == torrent_status::seeding);
 
 		if (st2.is_finished) break;
 
 		test_sleep(100);
 	}
 
-	TEST_CHECK(!tor2.status().is_seeding);
+	// torrent 2 should not be seeding yet, it should
+	// just be 50% finished
+	TEST_CHECK(!st2.is_seeding);
+	TEST_CHECK(st2.is_finished);
 
 	std::fill(priorities.begin(), priorities.end(), 1);
 	tor2.prioritize_pieces(priorities);
@@ -345,7 +335,7 @@ void test_transfer(session_settings& sett)
 	// this loop makes sure ses2 reconnects to the peer now that it's
 	// in download mode again. If this fails, the reconnect logic may
 	// not work or be inefficient
-	torrent_status st1 = tor1.status();
+	st1 = tor1.status();
 	st2 = tor2.status();
 	for (int i = 0; i < 130; ++i)
 	{
@@ -377,43 +367,57 @@ void test_transfer(session_settings& sett)
 		fprintf(stderr, "ses2 failed to reconnect to ses1!\n");
 	TEST_CHECK(st2.is_seeding);
 
-	// this allows shutting down the sessions in parallel
 	sp.push_back(ses1.abort());
 	sp.push_back(ses2.abort());
 }
 
-int test_main()
+TORRENT_TEST(priority)
 {
 	using namespace libtorrent;
-
-	udp_tracker_port = start_udp_tracker();
-	tracker_port = start_web_server();
-
-	session_settings sett;
-	sett.allow_multiple_connections_per_ip = false;
-	sett.ignore_limits_on_local_network = false;
-	sett.unchoke_slots_limit = 8;
-
-	test_transfer(sett);
-
-	sett.lazy_bitfields = true;
-
-	test_transfer(sett);
-	
-	error_code ec;
-	remove_all("tmp1_priorities", ec);
-	remove_all("tmp2_priorities", ec);
-	remove_all("tmp1_priorities_moved", ec);
-	remove_all("tmp2_priorities_moved", ec);
-
-	stop_udp_tracker();
-	stop_web_server();
-
-	// we have to clear them, session doesn't really support being destructed
-	// as a global destructor (for silly reasons)
-	sp.clear();
-
-	return 0;
+	settings_pack p;
+	test_transfer(p);
+	cleanup();
 }
 
+// test to set piece and file priority on a torrent that doesn't have metadata
+// yet
+TORRENT_TEST(no_metadata_file_prio)
+{
+	settings_pack pack;
+	lt::session ses(pack);
 
+	add_torrent_params addp;
+	addp.flags &= ~add_torrent_params::flag_paused;
+	addp.flags &= ~add_torrent_params::flag_auto_managed;
+	addp.info_hash = sha1_hash("abababababababababab");
+	addp.save_path = ".";
+	torrent_handle h = ses.add_torrent(addp);
+
+	h.file_priority(0, 0);
+	TEST_EQUAL(h.file_priority(0), 0);
+	h.file_priority(0, 1);
+	TEST_EQUAL(h.file_priority(0), 1);
+
+	ses.remove_torrent(h);
+}
+
+TORRENT_TEST(no_metadata_piece_prio)
+{
+	settings_pack pack;
+	lt::session ses(pack);
+
+	add_torrent_params addp;
+	addp.flags &= ~add_torrent_params::flag_paused;
+	addp.flags &= ~add_torrent_params::flag_auto_managed;
+	addp.info_hash = sha1_hash("abababababababababab");
+	addp.save_path = ".";
+	torrent_handle h = ses.add_torrent(addp);
+
+	// you can't set piece priorities before the metadata has been downloaded
+	h.piece_priority(2, 0);
+	TEST_EQUAL(h.piece_priority(2), 4);
+	h.piece_priority(2, 1);
+	TEST_EQUAL(h.piece_priority(2), 4);
+
+	ses.remove_torrent(h);
+}
