@@ -36,19 +36,24 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/random.hpp"
 #include "libtorrent/create_torrent.hpp"
 #include "libtorrent/alert_types.hpp"
+#include "libtorrent/entry.hpp"
 #include "libtorrent/bencode.hpp"
 
+#include <boost/make_shared.hpp>
+
 #include "test.hpp"
 #include "setup_transfer.hpp"
 
 using namespace libtorrent;
 namespace lt = libtorrent;
 
-boost::intrusive_ptr<torrent_info> generate_torrent()
+boost::shared_ptr<torrent_info> generate_torrent()
 {
 	file_storage fs;
-	fs.add_file("test_resume/tmp1", 128 * 1024 * 10);
-	libtorrent::create_torrent t(fs, 128 * 1024, 6);
+	fs.add_file("test_resume/tmp1", 128 * 1024 * 8);
+	fs.add_file("test_resume/tmp2", 128 * 1024);
+	fs.add_file("test_resume/tmp3", 128 * 1024);
+	lt::create_torrent t(fs, 128 * 1024, 6);
 
 	t.add_tracker("http://torrent_file_tracker.com/announce");
 	t.add_url_seed("http://torrent_file_url_seed.com/");
@@ -58,16 +63,17 @@ boost::intrusive_ptr<torrent_info> generate_torrent()
 	for (int i = 0; i < num; ++i)
 	{
 		sha1_hash ph;
-		for (int k = 0; k < 20; ++k) ph[k] = libtorrent::random();
+		for (int k = 0; k < 20; ++k) ph[k] = lt::random();
 		t.set_hash(i, ph);
 	}
 
 	std::vector<char> buf;
 	bencode(std::back_inserter(buf), t.generate());
-	return boost::intrusive_ptr<torrent_info>(new torrent_info(&buf[0], buf.size()));
+	return boost::make_shared<torrent_info>(&buf[0], buf.size());
 }
 
-std::vector<char> generate_resume_data(torrent_info* ti)
+std::vector<char> generate_resume_data(torrent_info* ti
+	, char const* file_priorities = "")
 {
 	entry rd;
 
@@ -95,8 +101,12 @@ std::vector<char> generate_resume_data(torrent_info* ti)
 	rd["last_download"] = 1350;
 	rd["last_upload"] = 1351;
 	rd["finished_time"] = 1352;
-	entry::list_type& file_prio = rd["file_priority"].list();
-	file_prio.push_back(entry(1));
+	if (file_priorities && file_priorities[0])
+	{
+		entry::list_type& file_prio = rd["file_priority"].list();
+		for (int i = 0; file_priorities[i]; ++i)
+			file_prio.push_back(entry(file_priorities[i] - '0'));
+	}
 
 	rd["piece_priority"] = std::string(ti->num_pieces(), '\x01');
 	rd["auto_managed"] = 0;
@@ -123,9 +133,10 @@ std::vector<char> generate_resume_data(torrent_info* ti)
 	return ret;
 }
 
-torrent_handle test_resume_flags(lt::session& ses, int flags)
+torrent_handle test_resume_flags(lt::session& ses, int flags
+	, char const* file_priorities = "1111", char const* resume_file_prio = "")
 {
-	boost::intrusive_ptr<torrent_info> ti = generate_torrent();
+	boost::shared_ptr<torrent_info> ti = generate_torrent();
 
 	add_torrent_params p;
 
@@ -139,36 +150,51 @@ torrent_handle test_resume_flags(lt::session& ses, int flags)
 	p.trackers.push_back("http://add_torrent_params_tracker.com/announce");
 	p.url_seeds.push_back("http://add_torrent_params_url_seed.com");
 
-	std::vector<char> rd = generate_resume_data(ti.get());
+	std::vector<char> rd = generate_resume_data(ti.get(), resume_file_prio);
 	p.resume_data.swap(rd);
 
 	p.max_uploads = 1;
 	p.max_connections = 2;
 	p.upload_limit = 3;
 	p.download_limit = 4;
-	p.file_priorities.push_back(2);
+
+	std::vector<boost::uint8_t> priorities_vector;
+	for (int i = 0; file_priorities[i]; ++i)
+		priorities_vector.push_back(file_priorities[i] - '0');
+
+	p.file_priorities = priorities_vector;
 
 	torrent_handle h = ses.add_torrent(p);
-	TEST_EQUAL(h.status().info_hash, ti->info_hash());
+	torrent_status s = h.status();
+	TEST_EQUAL(s.info_hash, ti->info_hash());
 	return h;
 }
 
 void default_tests(torrent_status const& s)
 {
-	TEST_EQUAL(s.last_scrape, 1349);
-	TEST_EQUAL(s.time_since_download, 1350);
-	TEST_EQUAL(s.time_since_upload, 1351);
-	TEST_EQUAL(s.active_time, 1339);
+	// allow some slack in the time stamps since they are reported as
+	// relative times. If the computer is busy while running the unit test
+	// or running under valgrind it may take several seconds
+	TEST_CHECK(s.last_scrape >= 1349);
+	TEST_CHECK(s.time_since_download >= 1350);
+	TEST_CHECK(s.time_since_upload >= 1351);
+	TEST_CHECK(s.active_time >= 1339);
+
+	TEST_CHECK(s.last_scrape < 1349 + 10);
+	TEST_CHECK(s.time_since_download < 1350 + 10);
+	TEST_CHECK(s.time_since_upload < 1351 + 10);
+	TEST_CHECK(s.active_time < 1339 + 10);
+
 	TEST_EQUAL(s.finished_time, 1352);
 	TEST_EQUAL(s.seeding_time, 1340);
 	TEST_EQUAL(s.added_time, 1347);
 	TEST_EQUAL(s.completed_time, 1348);
 }
 
-void test_piece_priorities()
+TORRENT_TEST(piece_priorities)
 {
-	session ses;
-	boost::intrusive_ptr<torrent_info> ti = generate_torrent();
+	lt::session ses;
+	boost::shared_ptr<torrent_info> ti = generate_torrent();
 	add_torrent_params p;
 	p.ti = ti;
 	p.save_path = ".";
@@ -178,16 +204,16 @@ void test_piece_priorities()
 	h.piece_priority(ti->num_pieces()-1, 0);
 
 	h.save_resume_data();
-	std::auto_ptr<alert> a = wait_for_alert(ses, save_resume_data_alert::alert_type);
+	alert const* a = wait_for_alert(ses, save_resume_data_alert::alert_type);
 
-	TEST_CHECK(a.get());
-	if (save_resume_data_alert* ra = alert_cast<save_resume_data_alert>(a.get()))
+	TEST_CHECK(a);
+	if (save_resume_data_alert const* ra = alert_cast<save_resume_data_alert>(a))
 	{
 		fprintf(stderr, "%s\n", ra->resume_data->to_string().c_str());
 		entry::string_type prios = (*ra->resume_data)["piece_priority"].string();
-		TEST_EQUAL(prios.size(), ti->num_pieces());
+		TEST_EQUAL(int(prios.size()), ti->num_pieces());
 		TEST_EQUAL(prios[0], '\0');
-		TEST_EQUAL(prios[1], '\x01');
+		TEST_EQUAL(prios[1], '\x04');
 		TEST_EQUAL(prios[ti->num_pieces()-1], '\0');
 
 		bencode(std::back_inserter(p.resume_data), *ra->resume_data);
@@ -197,17 +223,63 @@ void test_piece_priorities()
 
 	// now, make sure the piece priorities are loaded correctly
 	h = ses.add_torrent(p);
+
 	TEST_EQUAL(h.piece_priority(0), 0);
-	TEST_EQUAL(h.piece_priority(1), 1);
+	TEST_EQUAL(h.piece_priority(1), 4);
 	TEST_EQUAL(h.piece_priority(ti->num_pieces()-1), 0);
 }
 
-void test_file_prio()
+// TODO: test what happens when loading a resume file with both piece priorities
+// and file priorities (file prio should take presedence)
+
+// TODO: make sure a resume file only ever contain file priorities OR piece
+// priorities. Never both.
+
+// TODO: generally save 
+
+TORRENT_TEST(file_priorities_default)
+{
+	lt::session ses;
+	std::vector<int> file_priorities = test_resume_flags(ses, 0, "", "").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 4);
+	TEST_EQUAL(file_priorities[1], 4);
+	TEST_EQUAL(file_priorities[2], 4);
+}
+
+TORRENT_TEST(file_priorities_resume_seed_mode)
+{
+	// in share mode file priorities should always be 0
+	lt::session ses;
+	std::vector<int> file_priorities = test_resume_flags(ses,
+		add_torrent_params::flag_share_mode, "", "123").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 0);
+	TEST_EQUAL(file_priorities[1], 0);
+	TEST_EQUAL(file_priorities[2], 0);
+}
+
+TORRENT_TEST(file_priorities_seed_mode)
+{
+	// in share mode file priorities should always be 0
+	lt::session ses;
+	std::vector<int> file_priorities = test_resume_flags(ses,
+		add_torrent_params::flag_share_mode, "123", "").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 0);
+	TEST_EQUAL(file_priorities[1], 0);
+	TEST_EQUAL(file_priorities[2], 0);
+}
+
+TORRENT_TEST(zero_file_prio)
 {
 	fprintf(stderr, "test_file_prio\n");
 
-	session ses;
-	boost::intrusive_ptr<torrent_info> ti = generate_torrent();
+	lt::session ses;
+	boost::shared_ptr<torrent_info> ti = generate_torrent();
 	add_torrent_params p;
 	p.ti = ti;
 	p.save_path = ".";
@@ -245,8 +317,8 @@ void test_seed_mode(bool file_prio, bool pieces_have, bool piece_prio
 	fprintf(stderr, "test_seed_mode file_prio: %d pieces_have: %d piece_prio: %d\n"
 		, file_prio, pieces_have, piece_prio);
 
-	session ses;
-	boost::intrusive_ptr<torrent_info> ti = generate_torrent();
+	lt::session ses;
+	boost::shared_ptr<torrent_info> ti = generate_torrent();
 	add_torrent_params p;
 	p.ti = ti;
 	p.save_path = ".";
@@ -303,22 +375,144 @@ void test_seed_mode(bool file_prio, bool pieces_have, bool piece_prio
 	}
 }
 
-int test_main()
+TORRENT_TEST(seed_mode_file_prio)
 {
-	test_file_prio();
-
 	test_seed_mode(true, false, false);
+}
+
+TORRENT_TEST(seed_mode_piece_prio)
+{
 	test_seed_mode(false, true, false);
+}
+
+TORRENT_TEST(seed_mode_piece_have)
+{
 	test_seed_mode(false, false, true);
+}
+
+TORRENT_TEST(seed_mode_preserve)
+{
 	test_seed_mode(false, false, false);
+}
 
-	test_piece_priorities();
+TORRENT_TEST(resume_save_load)
+{
+	lt::session ses;
+	torrent_handle h = test_resume_flags(ses, 0, "123", "");
 
-	boost::scoped_ptr<lt::session> ses(new lt::session());
-	torrent_status s;
+	h.save_resume_data();
 
-	fprintf(stderr, "flags: 0\n");
-	s = test_resume_flags(*ses, 0).status();
+	save_resume_data_alert const* a = alert_cast<save_resume_data_alert>(
+		wait_for_alert(ses, save_resume_data_alert::alert_type
+		, "resume_save_load"));
+
+	TEST_CHECK(a);
+	if (a == NULL) return;
+
+	TEST_CHECK(a->resume_data);
+
+	entry& e = *a->resume_data.get();
+	entry::list_type& l = e["file_priority"].list();
+	entry::list_type::iterator i = l.begin();
+
+	TEST_EQUAL(l.size(), 3);
+	TEST_EQUAL(*i++, 1);
+	TEST_EQUAL(*i++, 2);
+	TEST_EQUAL(*i++, 3);
+}
+
+TORRENT_TEST(resume_save_load_resume)
+{
+	lt::session ses;
+	torrent_handle h = test_resume_flags(ses, 0, "", "123");
+
+	h.save_resume_data();
+
+	save_resume_data_alert const* a = alert_cast<save_resume_data_alert>(
+		wait_for_alert(ses, save_resume_data_alert::alert_type
+		, "resume_save_load"));
+
+	TEST_CHECK(a);
+	if (a == NULL) return;
+
+	TEST_CHECK(a->resume_data);
+
+	entry& e = *a->resume_data.get();
+	entry::list_type& l = e["file_priority"].list();
+	entry::list_type::iterator i = l.begin();
+
+	TEST_EQUAL(l.size(), 3);
+	TEST_EQUAL(*i++, 1);
+	TEST_EQUAL(*i++, 2);
+	TEST_EQUAL(*i++, 3);
+}
+
+TORRENT_TEST(file_priorities_resume_override)
+{
+	// make sure that an empty file_priorities vector in add_torrent_params won't
+	// override the resume data file priorities, even when override resume data
+	// flag is set.
+	lt::session ses;
+	std::vector<int> file_priorities = test_resume_flags(ses,
+		add_torrent_params::flag_override_resume_data, "", "123").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 1);
+	TEST_EQUAL(file_priorities[1], 2);
+	TEST_EQUAL(file_priorities[2], 3);
+}
+
+TORRENT_TEST(file_priorities_resume)
+{
+	lt::session ses;
+	std::vector<int> file_priorities = test_resume_flags(ses, 0, "", "123").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 1);
+	TEST_EQUAL(file_priorities[1], 2);
+	TEST_EQUAL(file_priorities[2], 3);
+}
+
+TORRENT_TEST(file_priorities1)
+{
+	lt::session ses;
+	std::vector<int> file_priorities = test_resume_flags(ses, 0, "010").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 0);
+	TEST_EQUAL(file_priorities[1], 1);
+	TEST_EQUAL(file_priorities[2], 0);
+
+//#error save resume data and assert the file priorities are preserved
+}
+
+TORRENT_TEST(file_priorities2)
+{
+	lt::session ses;
+	std::vector<int> file_priorities = test_resume_flags(ses, 0, "123").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 1);
+	TEST_EQUAL(file_priorities[1], 2);
+	TEST_EQUAL(file_priorities[2], 3);
+}
+
+TORRENT_TEST(file_priorities3)
+{
+	lt::session ses;
+	std::vector<int> file_priorities = test_resume_flags(ses, 0, "4321").file_priorities();
+
+	TEST_EQUAL(file_priorities.size(), 3);
+	TEST_EQUAL(file_priorities[0], 4);
+	TEST_EQUAL(file_priorities[1], 3);
+	TEST_EQUAL(file_priorities[2], 2);
+}
+
+TORRENT_TEST(plain)
+{
+	lt::session ses;
+
+	torrent_status s = test_resume_flags(ses, 0).status();
 	default_tests(s);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
@@ -335,10 +529,12 @@ int test_main()
 	TEST_EQUAL(s.ip_filter_applies, false);
 	TEST_EQUAL(s.connections_limit, 1345);
 	TEST_EQUAL(s.uploads_limit, 1346);
+}
 
-	fprintf(stderr, "flags: use_resume_save_path\n");
-	ses.reset(new lt::session());
-	s = test_resume_flags(*ses, add_torrent_params::flag_use_resume_save_path).status();
+TORRENT_TEST(use_resume_save_path)
+{
+	lt::session ses;
+	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_use_resume_save_path).status();
 	default_tests(s);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\resume_data save_path");
@@ -355,10 +551,13 @@ int test_main()
 	TEST_EQUAL(s.ip_filter_applies, false);
 	TEST_EQUAL(s.connections_limit, 1345);
 	TEST_EQUAL(s.uploads_limit, 1346);
+}
 
-	fprintf(stderr, "flags: override_resume_data\n");
-	ses.reset(new lt::session());
-	s = test_resume_flags(*ses, add_torrent_params::flag_override_resume_data
+TORRENT_TEST(override_resume_data)
+{
+	lt::session ses;
+	torrent_status s = test_resume_flags(ses
+		, add_torrent_params::flag_override_resume_data
 		| add_torrent_params::flag_paused).status();
 
 	default_tests(s);
@@ -377,10 +576,12 @@ int test_main()
 	TEST_EQUAL(s.ip_filter_applies, false);
 	TEST_EQUAL(s.connections_limit, 2);
 	TEST_EQUAL(s.uploads_limit, 1);
+}
 
-	fprintf(stderr, "flags: seed_mode\n");
-	ses.reset(new lt::session());
-	s = test_resume_flags(*ses, add_torrent_params::flag_override_resume_data
+TORRENT_TEST(seed_mode)
+{
+	lt::session ses;
+	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_override_resume_data
 		| add_torrent_params::flag_seed_mode).status();
 	default_tests(s);
 #ifdef TORRENT_WINDOWS
@@ -398,10 +599,12 @@ int test_main()
 	TEST_EQUAL(s.ip_filter_applies, false);
 	TEST_EQUAL(s.connections_limit, 2);
 	TEST_EQUAL(s.uploads_limit, 1);
+}
 
-	fprintf(stderr, "flags: upload_mode\n");
-	ses.reset(new lt::session());
-	s = test_resume_flags(*ses, add_torrent_params::flag_upload_mode).status();
+TORRENT_TEST(upload_mode)
+{
+	lt::session ses;
+	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_upload_mode).status();
 	default_tests(s);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
@@ -418,10 +621,13 @@ int test_main()
 	TEST_EQUAL(s.ip_filter_applies, false);
 	TEST_EQUAL(s.connections_limit, 1345);
 	TEST_EQUAL(s.uploads_limit, 1346);
+}
 
-	fprintf(stderr, "flags: share_mode\n");
-	ses.reset(new lt::session());
-	s = test_resume_flags(*ses, add_torrent_params::flag_override_resume_data
+TORRENT_TEST(share_mode)
+{
+	lt::session ses;
+	torrent_status s = test_resume_flags(ses
+		, add_torrent_params::flag_override_resume_data
 		| add_torrent_params::flag_share_mode).status();
 	default_tests(s);
 #ifdef TORRENT_WINDOWS
@@ -439,11 +645,13 @@ int test_main()
 	TEST_EQUAL(s.ip_filter_applies, false);
 	TEST_EQUAL(s.connections_limit, 2);
 	TEST_EQUAL(s.uploads_limit, 1);
+}
 
+TORRENT_TEST(auto_managed)
+{
+	lt::session ses;
 	// resume data overrides the auto-managed flag
-	fprintf(stderr, "flags: auto_managed\n");
-	ses.reset(new lt::session());
-	s = test_resume_flags(*ses, add_torrent_params::flag_auto_managed).status();
+	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_auto_managed).status();
 	default_tests(s);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
@@ -460,11 +668,13 @@ int test_main()
 	TEST_EQUAL(s.ip_filter_applies, false);
 	TEST_EQUAL(s.connections_limit, 1345);
 	TEST_EQUAL(s.uploads_limit, 1346);
+}
 
+TORRENT_TEST(paused)
+{
+	lt::session ses;
 	// resume data overrides the paused flag
-	fprintf(stderr, "flags: paused\n");
-	ses.reset(new lt::session());
-	s = test_resume_flags(*ses, add_torrent_params::flag_paused).status();
+	torrent_status s = test_resume_flags(ses, add_torrent_params::flag_paused).status();
 	default_tests(s);
 #ifdef TORRENT_WINDOWS
 	TEST_EQUAL(s.save_path, "c:\\add_torrent_params save_path");
@@ -482,10 +692,17 @@ int test_main()
 	TEST_EQUAL(s.connections_limit, 1345);
 	TEST_EQUAL(s.uploads_limit, 1346);
 
+	// TODO: test all other resume flags here too. This would require returning
+	// more than just the torrent_status from test_resume_flags. Also http seeds
+	// and trackers for instance
+}
+
+TORRENT_TEST(url_seed_resume_data)
+{
 	// merge url seeds with resume data
 	fprintf(stderr, "flags: merge_resume_http_seeds\n");
-	ses.reset(new lt::session());
-	torrent_handle h = test_resume_flags(*ses,
+	lt::session ses;
+	torrent_handle h = test_resume_flags(ses,
 		add_torrent_params::flag_merge_resume_http_seeds);
 	std::set<std::string> us = h.url_seeds();
 	std::set<std::string> ws = h.http_seeds();
@@ -496,19 +713,22 @@ int test_main()
 	TEST_EQUAL(std::count(us.begin(), us.end()
 		, "http://torrent_file_url_seed.com/"), 1);
 	TEST_EQUAL(std::count(us.begin(), us.end()
-		, "http://resume_data_url_seed.com"), 1);
+		, "http://resume_data_url_seed.com/"), 1);
 
 	TEST_EQUAL(ws.size(), 1);
 	TEST_EQUAL(std::count(ws.begin(), ws.end()
 		, "http://resume_data_http_seed.com"), 1);
+}
 
+TORRENT_TEST(resume_override_torrent)
+{
 	// resume data overrides the .torrent_file
 	fprintf(stderr, "flags: no merge_resume_http_seed\n");
-	ses.reset(new lt::session());
-	h = test_resume_flags(*ses,
+	lt::session ses;
+	torrent_handle h = test_resume_flags(ses,
 		add_torrent_params::flag_merge_resume_trackers);
-	us = h.url_seeds();
-	ws = h.http_seeds();
+	std::set<std::string> us = h.url_seeds();
+	std::set<std::string> ws = h.http_seeds();
 
 	TEST_EQUAL(ws.size(), 1);
 	TEST_EQUAL(std::count(ws.begin(), ws.end()
@@ -516,12 +736,6 @@ int test_main()
 
 	TEST_EQUAL(us.size(), 1);
 	TEST_EQUAL(std::count(us.begin(), us.end()
-		, "http://resume_data_url_seed.com"), 1);
-
-	// TODO: 2 test all other resume flags here too. This would require returning
-	// more than just the torrent_status from test_resume_flags. Also http seeds
-	// and trackers for instance
-	return 0;
+		, "http://resume_data_url_seed.com/"), 1);
 }
 
-
